using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;
using UnityEngine.UI;

public class BossGiantIdle : StateMachineBehaviour
{
    float chaseRange = 25.0f;
    List<Transform> playersTransform = new List<Transform>();
    NavMeshAgent navMeshAgent;
    int closestPlayerIndex = 0;

    //OnStateEnter is called when a transition starts and the state machine starts to evaluate this state
    override public void OnStateEnter(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        navMeshAgent = animator.GetComponent<NavMeshAgent>();
        GameObject[] playersGameObject = GameObject.FindGameObjectsWithTag("player");
        foreach (GameObject player in playersGameObject)
        {
            playersTransform.Add(player.transform);
        }
    }

    //OnStateUpdate is called on each Update frame between OnStateEnter and OnStateExit callbacks
    override public void OnStateUpdate(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        float shortestDistance = float.MaxValue;
        for (int i = 0; i < playersTransform.Count; i++)
        {
            //float playerDistance = Vector3.Distance(playersTransform[i].position, animator.transform.position);
            NavMeshPath Path = new();
            if(navMeshAgent.CalculatePath(playersTransform[i].position, Path))
            {
                float playerDistance = PathLength(Path);

                if (playerDistance < shortestDistance)
                {
                    closestPlayerIndex = i;
                    shortestDistance = playerDistance;
                }
            }
        }

        BossGiant bossInstance = animator.GetComponent<BossGiant>();
        if (bossInstance.stats.currentHp < (0.6f * bossInstance.stats.maxHp))
        {
            bossInstance.isIn2ndPhase = true;
            animator.SetBool("BossIsInSecondPhase", true);
            animator.SetBool("IsSpinning", true);
        }
        else if(shortestDistance < chaseRange)
        {
            animator.SetBool("IsChasing", true);
            animator.SetInteger("ClosestPlayerIndex", closestPlayerIndex);
        }
    }

    //OnStateExit is called when a transition ends and the state machine finishes evaluating this state
    override public void OnStateExit(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
    }

    // OnStateMove is called right after Animator.OnAnimatorMove()
    //override public void OnStateMove(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    //{
    //    // Implement code that processes and affects root motion
    //}

    // OnStateIK is called right after Animator.OnAnimatorIK()
    //override public void OnStateIK(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    //{
    //    // Implement code that sets up animation IK (inverse kinematics)
    //}


    // Reference : 
    // https://docs.unity3d.com/540/Documentation/ScriptReference/NavMeshPath-corners.html
    float PathLength(NavMeshPath path)
    {
        if (path.corners.Length < 2)
            return 0;

        Vector3 previousCorner = path.corners[0];
        float lengthSoFar = 0.0F;
        int i = 1;
        while (i < path.corners.Length)
        {
            Vector3 currentCorner = path.corners[i];
            lengthSoFar += Vector3.Distance(previousCorner, currentCorner);
            previousCorner = currentCorner;
            i++;
        }
        return lengthSoFar;
    }
}
