using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class BossGiantRock : MonoBehaviour
{
    // G�re le spawn du rocher qui tombe sur un joueur au hasard
    // G�re le spawn de l'ombre qui est un indicateur de o� le rocher va tomber
    private float rockSpawnHeight = 25f;
    private float maxTimeOnGround = 2.5f;
    private float minDistanceOffGroundStartTimer = 5f;
    private float fadeOutSpeed = 0.25f;

    private float timer;

    private Light lightRock;
    private Camera playerCamera;
    private int rockLayerMask;
    private bool isCurrentlyObstructing;
    List<Transform> playersTransform = new List<Transform>();

    public GameObject rock;
    public GameObject rockShadowIndicator;
    // Start is called before the first frame update
    void Start()
    {
        // Init des variables necessaires dans Update()
        timer = 0f;
        isCurrentlyObstructing = false;
        rockLayerMask = LayerMask.GetMask("rock");
        lightRock = FindObjectOfType<Light>();
        playerCamera = FindObjectOfType<Camera>();

        GameObject[] playersGameObject = GameObject.FindGameObjectsWithTag("player");
        foreach (GameObject player in playersGameObject)
        {
            playersTransform.Add(player.transform);
        }
        Transform randomPlayer = playersTransform[ScriptGenerationDonjon.randomPersonal.Next(0, playersTransform.Count)];

        // Spawn du rocher physique
        Vector3 rockSpawnPosition = randomPlayer.position;
        rockSpawnPosition.y += rockSpawnHeight;
        rock.transform.position = rockSpawnPosition;
        rock.transform.rotation = new Quaternion((float) ScriptGenerationDonjon.randomPersonal.NextDouble(), (float)ScriptGenerationDonjon.randomPersonal.NextDouble(), (float)ScriptGenerationDonjon.randomPersonal.NextDouble(), (float)ScriptGenerationDonjon.randomPersonal.NextDouble());

        // Spawn de l'indicateur du rocher (Projection de la position du rocher sur l'axe Joueur<->Lumiere)
        //Vector3 indicatorSpawnDirection = -(lightRock.transform.forward);
        //Vector3 randomPlayerFeet = randomPlayer.position;
        //randomPlayerFeet.y = 0;
        //Vector3 indicatorSpawnPosition = randomPlayerFeet + indicatorSpawnDirection * rockSpawnHeight;
        rockShadowIndicator.transform.position = randomPlayer.position;

    }

    // Update is called once per frame
    void Update()
    {
        bool obstructsPlayerView = false;
        Color fadeOut;
        foreach (Transform player in playersTransform)
        { 
            Ray ray = new Ray(player.position, playerCamera.transform.position - player.position);
            obstructsPlayerView = obstructsPlayerView || Physics.Raycast(ray, Mathf.Infinity, rockLayerMask);
        }

        if (rock.transform.position.y < minDistanceOffGroundStartTimer)
        {
            if(isCurrentlyObstructing)
            {
                isCurrentlyObstructing = false;
                SetAlphaOfRock(1f);
            }

            timer += Time.deltaTime;

            // Supprimer l'ombre, fadeout et de-spawn le rocher
            rockShadowIndicator.SetActive(false);

            fadeOut = rock.GetComponent<Renderer>().material.color;
            fadeOut.a -= (fadeOutSpeed * Time.deltaTime);
            rock.GetComponent<Renderer>().material.color = fadeOut;
        }
        else if(obstructsPlayerView)
        {
            isCurrentlyObstructing = true;
            SetAlphaOfRock(0.2f);
        }
        else if(isCurrentlyObstructing) // le rocher ne gene plus l'affichage mais est toujours transparent
        {
            isCurrentlyObstructing = false;
            SetAlphaOfRock(1f);
        }

        if(timer > maxTimeOnGround)
        {
            Destroy(this.gameObject);
        }
    }


    private void SetAlphaOfRock(float alpha)
    {
        Color fadeOut;
        fadeOut = rock.GetComponent<Renderer>().material.color;
        fadeOut.a = alpha;
        rock.GetComponent<Renderer>().material.color = fadeOut;
    }
}
