using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.AI;
public class BossGiantRotateAttack : StateMachineBehaviour
{
    float timer;
    float rotateTimeSeconds = 10.0f;
    float roomMaxLen = 150.0f;
    float bossCollisionWidth = 3.0f;
    float oldSpeed;
    float newSpeed = 15.0f;
    float oldAcceleration;
    float newAcceleration = 100000;

    NavMeshAgent navMeshAgent;
    List<Transform> playersTransform = new List<Transform>();
    Transform closestPlayer;

    int playerLayerMask;
    //int rockLayerMask;
    Vector3 currentDestination;
    Vector3 currentDirection;
    Vector3 currentNormalVect;
    RaycastHit rayHitInfo;

    private GameObject lastWallHit;

    public GameObject rockPrefab;

    private BossGiant bossInstance;

    //OnStateEnter is called when a transition starts and the state machine starts to evaluate this state
    override public void OnStateEnter(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        animator.GetComponent<BossGiant>().rotateParticleSys.Play();

        int closestPlayerIndex = animator.GetInteger("ClosestPlayerIndex");
        timer = 0.0f;

        bossInstance = animator.GetComponent<BossGiant>();
        bossInstance.canDamageRotate = true;

        navMeshAgent = animator.GetComponent<NavMeshAgent>();
        oldSpeed = navMeshAgent.speed;
        oldAcceleration = navMeshAgent.acceleration;
        navMeshAgent.speed = newSpeed;
        navMeshAgent.acceleration = newAcceleration;

        GameObject[] playersGameObject = GameObject.FindGameObjectsWithTag("player");
        foreach (GameObject player in playersGameObject)
        {
            playersTransform.Add(player.transform);
        }

        closestPlayer = playersTransform[closestPlayerIndex];
        // Pour rappel vecteur de A vers B vaut (B.x - A.x, B.y - A.y, ...)
        Vector3 closestPlayerDirection = closestPlayer.position - animator.transform.position;
        currentDirection = closestPlayerDirection;
        // Seul le mouvement en X et en Z nous interesse dans ce cas
        currentDirection.y = 0;

        // Correspond aux murs
        playerLayerMask = LayerMask.GetMask("chest");
        //rockLayerMask = LayerMask.GetMask("rock");

        Ray ray = new Ray(new Vector3(animator.transform.position.x, 1, animator.transform.position.z), currentDirection);
        Debug.DrawRay(new Vector3(animator.transform.position.x, 1, animator.transform.position.z), currentDirection, Color.green, 5);
        if (Physics.Raycast(ray, out rayHitInfo, roomMaxLen, playerLayerMask))
        {
            currentDestination = rayHitInfo.point;
            currentNormalVect = rayHitInfo.normal;
            navMeshAgent.SetDestination(currentDestination);
        }
        else
        {
            // Ne devrait jamais arriver mais permet au jeu de continuer si jamais Raycast echoue
            navMeshAgent.SetDestination(closestPlayer.position);
        }
    }
    //OnStateUpdate is called on each Update frame between OnStateEnter and OnStateExit callbacks
    override public void OnStateUpdate(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        timer += Time.deltaTime;
        if (timer >= rotateTimeSeconds)
        {
            animator.SetBool("IsSpinning", false);
        }

        if (Vector3.Distance(animator.transform.position, currentDestination) < bossCollisionWidth)
        {
            currentDirection = Vector3.Reflect(currentDirection, currentNormalVect);

            Ray ray = new Ray(new Vector3(animator.transform.position.x, 1, animator.transform.position.z), currentDirection);

            bool mustInstantiateRock = false;

            if (Physics.Raycast(ray, out rayHitInfo, roomMaxLen, playerLayerMask))
            {
                if(lastWallHit == null || rayHitInfo.collider.gameObject.name != lastWallHit.name)
                    mustInstantiateRock = true;
                currentDestination = rayHitInfo.point;
                currentNormalVect = rayHitInfo.normal;
                navMeshAgent.SetDestination(currentDestination);
            }
            else
            {
                // Ne devrait jamais arriver mais permet au jeu de continuer si jamais Raycast echoue
                navMeshAgent.SetDestination(closestPlayer.position);
            }

            BossGiant bossInstance = animator.GetComponent<BossGiant>();
            if (bossInstance.stats.currentHp < (0.3f * bossInstance.stats.maxHp) && mustInstantiateRock)
            {
                GameObject rock = Instantiate(rockPrefab);
            }
        }
    }

    //OnStateExit is called when a transition ends and the state machine finishes evaluating this state
    override public void OnStateExit(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        // Empecher que le boss continue de bouger en sortant de cette animation
        navMeshAgent.speed = oldSpeed;
        navMeshAgent.acceleration = oldAcceleration;
        navMeshAgent.SetDestination(animator.transform.position);

        bossInstance.canDamageRotate = false;

        animator.GetComponent<BossGiant>().rotateParticleSys.Stop(true, ParticleSystemStopBehavior.StopEmitting);
    }

    // OnStateMove is called right after Animator.OnAnimatorMove()
    //override public void OnStateMove(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    //{
    //    // Implement code that processes and affects root motion
    //}

    // OnStateIK is called right after Animator.OnAnimatorIK()
    //override public void OnStateIK(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    //{
    //    // Implement code that sets up animation IK (inverse kinematics)
    //}
}
