using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;
public class BossSlimeChase : StateMachineBehaviour
{
    /*
     * Les parties de rcupration des donnes relatives au NavMeshAgent et liste des joueurs
     * sont redondantes, on pourrait crer un GameObject , stocker les informations dedans et y accder
     * Cela ncessite une coordination entre les branches et peut tre source de dpendances et donc de bugs
     */

    private BossSlime bossInstance;

    private int closestPlayerIndex;
    NavMeshAgent navMeshAgent;
    List<Transform> playersTransform = new List<Transform>();

    private float maxChaseTime = 10f;
    private float chaseTimer;

    //OnStateEnter is called when a transition starts and the state machine starts to evaluate this state
    override public void OnStateEnter(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        bossInstance = animator.GetComponent<BossSlime>();
        bossInstance.canDealDamage = true;

        navMeshAgent = animator.GetComponent<NavMeshAgent>();
        closestPlayerIndex = animator.GetInteger("ClosestPlayerIndex");

        GameObject[] playersGameObject = GameObject.FindGameObjectsWithTag("player");
        foreach (GameObject player in playersGameObject)
        {
            playersTransform.Add(player.transform);
        }

        navMeshAgent.SetDestination(playersTransform[closestPlayerIndex].position);
        chaseTimer = 0f;
    }

    //OnStateUpdate is called on each Update frame between OnStateEnter and OnStateExit callbacks
    override public void OnStateUpdate(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        if (bossInstance.stats.currentHp < 0.75f * bossInstance.stats.maxHp)
        {
            animator.SetBool("IsInSecondPhase", true);
            return;
        }
        navMeshAgent.SetDestination(playersTransform[closestPlayerIndex].position);

        chaseTimer += Time.deltaTime;
        if (chaseTimer >= maxChaseTime)
        {
            chaseTimer = 0;
            bossInstance.particleSys.Stop(false, ParticleSystemStopBehavior.StopEmittingAndClear);
            animator.SetTrigger("IsStunned");
        }
    }

    //OnStateExit is called when a transition ends and the state machine finishes evaluating this state
    override public void OnStateExit(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        bossInstance.canDealDamage = false;
    }

    // OnStateMove is called right after Animator.OnAnimatorMove()
    //override public void OnStateMove(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    //{
    //    // Implement code that processes and affects root motion
    //}

    // OnStateIK is called right after Animator.OnAnimatorIK()
    //override public void OnStateIK(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    //{
    //    // Implement code that sets up animation IK (inverse kinematics)
    //}
}
