using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;
public class BossSlimeChaseAndFire : StateMachineBehaviour
{
    /*
     * Les parties de rcupration des donnes relatives au NavMeshAgent et liste des joueurs
     * sont redondantes, on pourrait crer un GameObject , stocker les informations dedans et y accder
     * Cela ncessite une coordination entre les branches et peut tre source de dpendances et donc de bugs
     */

    private string animationTagFireZones = "SpawnsFireZones";
    private BossSlime bossInstance;

    private int closestPlayerIndex;
    NavMeshAgent navMeshAgent;
    List<Transform> playersTransform = new List<Transform>();

    private float maxChaseTime = 10f;
    private float chaseTimer;

    private float fireTimer;
    private float fireZoneCooldown = 0.5f;

    public GameObject slimeBossPrefab;
    private int maxNumberOfDivides = 2;
    //OnStateEnter is called when a transition starts and the state machine starts to evaluate this state
    override public void OnStateEnter(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        bossInstance = animator.GetComponent<BossSlime>();
        bossInstance.particleSys.Play();
        bossInstance.canDealDamage = true;

        navMeshAgent = animator.GetComponent<NavMeshAgent>();
        closestPlayerIndex = animator.GetInteger("ClosestPlayerIndex");

        GameObject[] playersGameObject = GameObject.FindGameObjectsWithTag("player");
        foreach (GameObject player in playersGameObject)
        {
            playersTransform.Add(player.transform);
        }

        navMeshAgent.SetDestination(playersTransform[closestPlayerIndex].position);

        fireTimer = 0f;
        chaseTimer = 0f;
    }

    //OnStateUpdate is called on each Update frame between OnStateEnter and OnStateExit callbacks
    override public void OnStateUpdate(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        if ((bossInstance.stats.currentHp < 0.5f * bossInstance.stats.maxHp) && (bossInstance.numberOfDivides < maxNumberOfDivides))
        {
            // Resize du slime actuel
            animator.transform.localScale /= 2.0f;
            bossInstance.particleSys.transform.localScale /= 2.0f;
            bossInstance.stats.maxHp = bossInstance.stats.currentHp;
            bossInstance.numberOfDivides += 1;

            // Spawn d'un autre slime
            GameObject newSlimeGO = Instantiate(slimeBossPrefab);
            newSlimeGO.transform.localScale = animator.transform.localScale;
            BossSlime newBossClassObj = newSlimeGO.GetComponent<BossSlime>();
            newBossClassObj.EnemyInit();
            newBossClassObj.particleSys.transform.localScale = bossInstance.particleSys.transform.localScale;

            Vector3 newSlimePos = animator.transform.position;
            newSlimePos.y += 3;
            newSlimeGO.transform.position = newSlimePos;
            newBossClassObj.stats.currentHp = bossInstance.stats.currentHp;
            newBossClassObj.stats.maxHp = bossInstance.stats.maxHp;
            newBossClassObj.SetNumberOfDivides(bossInstance.numberOfDivides);
            newBossClassObj.canDealDamage = true;
            return;
        }

        // Edition du code de 'Inwithling' du 13 aout 2020 post sur le forum : 
        // https://forum.unity.com/threads/how-to-find-animation-clip-length.465751/
        var state = animator.GetCurrentAnimatorStateInfo(0);
        bool animationIsFinished = state.IsTag(animationTagFireZones) && state.normalizedTime >= 1;

        fireTimer += Time.deltaTime;
        if (animationIsFinished && (fireTimer >= fireZoneCooldown))
        {
            fireTimer = 0f;
            GameObject fireZone = Instantiate(bossInstance.fireZonePrefab);
            fireZone.transform.position = animator.transform.position;
        }

        chaseTimer += Time.deltaTime;
        if (chaseTimer >= maxChaseTime)
        {
            chaseTimer = 0;
            bossInstance.particleSys.Stop(false, ParticleSystemStopBehavior.StopEmittingAndClear);
            animator.SetTrigger("IsStunned");
        }

        navMeshAgent.SetDestination(playersTransform[closestPlayerIndex].position);
    }

    //OnStateExit is called when a transition ends and the state machine finishes evaluating this state
    override public void OnStateExit(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    {
        bossInstance.canDealDamage = false;
    }

    // OnStateMove is called right after Animator.OnAnimatorMove()
    //override public void OnStateMove(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    //{
    //    // Implement code that processes and affects root motion
    //}

    // OnStateIK is called right after Animator.OnAnimatorIK()
    //override public void OnStateIK(Animator animator, AnimatorStateInfo stateInfo, int layerIndex)
    //{
    //    // Implement code that sets up animation IK (inverse kinematics)
    //}
}
