using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using TMPro;
using UnityEditor;

public abstract class Enemy : MonoBehaviour
{
    public Animator animator;
    public Transform attackPoint;
    public LayerMask playersLayers;
    public GameObject[] players;
    public GameObject[] weapons;

    public GameObject currentRoom;

    public Vector3 initPosition;

    public float coolDown;
    public bool attackAvailable;
    
    
    [SerializeField] public EnemyStatistics stats;
    [SerializeField] public GameObject healthBar;
    public Transform prefabSpawnPoint;
    [SerializeField] public GameObject explosionPrefab;

    // SFX
    public AudioSource audioSrc;
    public List<AudioClip> sfxTookAHit;
    void Start()
    {
        initPosition = gameObject.transform.position;
        EnemyInit();
    }

    public abstract void EnemyInit();

    public abstract IEnumerator Attack();
    
    public abstract IEnumerator StartCoolDown(float coolDown, int idAttack);


    
    /*
     * Fonction qui fait perdre des points de vie à l'ennemi. 
     */
    public void TakeDamage(int damageAbility, float damageAttack)
    {
        stats.currentHp -= (damageAbility - stats.defense > 0) ? (int) ((damageAttack/100.0) * (damageAbility - stats.defense))  : 0;
        healthBar.GetComponent<Slider>().value = stats.currentHp;   
        animator.SetTrigger("GotHit");
        Instantiate(explosionPrefab, prefabSpawnPoint.position, prefabSpawnPoint.rotation);
        audioSrc.PlayOneShot(sfxTookAHit[ScriptGenerationDonjon.randomPersonal.Next(0, sfxTookAHit.Count)], 0.1f);
        if (stats.currentHp <= 0)
            Die();
    }
    
    public void dropReward()
    {
        SpawnerObject spawner = GameObject.Find("GameMaster").GetComponent<SpawnerObject>();
        if(ScriptGenerationDonjon.randomPersonal.NextDouble() > 0.5f)
        {
            int nbCoins = ScriptGenerationDonjon.randomPersonal.Next(1,5);
            spawner.dropCoins(200,nbCoins, transform.position, gameObject);
        }

        if(ScriptGenerationDonjon.randomPersonal.NextDouble() > 0.95f)
            spawner.dropHealthPotion(200,transform.position, gameObject);
    }

    /*
     * Fonction qui fait mourir l'ennemi.
     */
    public virtual void Die()
    {
        GameObject.Find("GameMaster").GetComponent<ScriptGenerationDonjon>().EnemyDead();
        healthBar.GetComponent<Slider>().value = 0;
        // Déclenche l'animation de mort
        animator.SetBool("IsDead", true);
        // Désactive TOUS les Colliders du mort
        Collider[] colliders;
        colliders = gameObject.GetComponentsInChildren<Collider>();
        foreach(Collider c in colliders)
            c.enabled = false;
        
        // Désactive l'entité
        this.enabled = false;
        // Détruit le joueur
        Destroy(this.gameObject.GetComponent<BoxCollider>());

        Destroy(this.gameObject, 2);

        if (Lobby.isMainPlayer)
            GetComponent<EnemySend>().RemoveFromLobby();
        
        dropReward();
    }


    void Update()
    {
        if(gameObject.transform.position.y < - 5)
            Die();
    }
}
