using Newtonsoft.Json;
using System.Collections;
using System.Collections.Generic;
using Script.Statistics;
using TMPro;
using Unity.VisualScripting;
using UnityEngine;
using UnityEngine.InputSystem;
using UnityEngine.UI;
public class MeleeMinion : Enemy
{
    public List<string> animations;
    private int nbAttacks = 0;    

    public void FixedUpdate()
    {
        // attaque lorsqu'il détecte des joueurs dans sa portée d'attaque
        Collider[] hitEnemies = Physics.OverlapSphere(attackPoint.position, stats.attackRange, playersLayers);
        if (hitEnemies.Length != 0) StartCoroutine(Attack());
        healthBar.GetComponent<Slider>().value = stats.currentHp;

    }
    
    public override void EnemyInit()
    {
        stats = ScriptableObject.CreateInstance<MeleeMinionStatistics>();
        stats.init();
        healthBar.GetComponent<Slider>().maxValue = stats.maxHp;
        healthBar.GetComponent<Slider>().value = stats.currentHp;
        coolDown = 50f;
        attackAvailable = true;
        animations.Add("AutoAttacking");
    }

    public override IEnumerator Attack()
    {
        if (!attackAvailable)
        {
            yield break;
        }
        // Déclenche l'animation d'attaque
        int nbAnimations = animations.Count;
        animator.SetTrigger(animations[nbAttacks++ % nbAnimations]);
        StartCoroutine(StartCoolDown(stats.attackSpeed, 1));
        // Détecte les ennemis à portée d'attaque
        Collider[] hitEnemies = Physics.OverlapSphere(attackPoint.position, stats.attackRange, playersLayers);
        // Infliger des dégâts aux ennemis au bout de .2f s.
        yield return new WaitForSeconds(.2f);

        foreach (Collider enemy in hitEnemies)
        {
            /* Le BoxCollider représente la HitBox */
            if (enemy is BoxCollider)
            {
                if (enemy != null)
                {
                    enemy.GetComponent<Players>().TakeDamage(stats.clickDmgAbility, stats.power);
                    KnockBacking(enemy);
                }
            }
        }
    }

    public override IEnumerator StartCoolDown(float coolDown, int idAttack)
    {
        attackAvailable = false;
        yield return new WaitForSeconds(coolDown);
        attackAvailable = true;
    }

    private void KnockBacking(Collider enemy)
    {
        Rigidbody enemyRB = enemy.GetComponent<Rigidbody>();
        //enemyRB.isKinematic = false;
        Vector3 difference = enemyRB.transform.position - transform.position;
        difference = difference.normalized * stats.thrust;
        enemyRB.AddForce(difference, ForceMode.Impulse);
        StartCoroutine(KnockCo(enemyRB));
    }
    private IEnumerator KnockCo(Rigidbody enemy)
    {
        if (enemy != null)
        {
            yield return new WaitForSeconds(stats.knockTime);
            if(enemy != null)
                enemy.velocity = Vector3.zero;
            //enemy.isKinematic = true;
        }
    }
}
