using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;
using UnityEngine.UI;
using UnityEngine.SceneManagement;
using Newtonsoft.Json;
using UnityEngine.Networking;

public class EndScene : MonoBehaviour
{

    // Ecran de fin
    private Transform screen;

    //Caméra de la scène
    private Camera EndSceneCamera;

    // Bouton pour quitter
    private Transform button;

    // Start is called before the first frame update
    IEnumerator Start()
    {
        if (SceneManager.GetActiveScene().name == "EndScene")
        {
            screen = GameObject.Find("EndScreen").transform;

            // A chaque frame, on diminue petit à petit l'alpha de la couleur de l'écran pour un fondu du noir à la scène
            for (float i = 0; i <= 1; i += Time.deltaTime)
            {
                screen.GetComponent<Image>().color = new Color(0, 0, 0, 1 - i);
                yield return null;
            }
            // On fixe la valeur de l'alpha à 0
            // A la sortie de boucle, on est pas sûr que l'alpha soit bien à 0, donc on fixe la valeur manuellement
            screen.GetComponent<Image>().color = new Color(0, 0, 0, 0);

            StartCoroutine(UpdateStats());

            Init();
        }
    }

    void Init()
    {
        //lancement de l'animation de la caméra
        EndSceneCamera = GameObject.Find("Main Camera").GetComponent<Camera>();
        EndSceneCamera.GetComponent<Animation>().Play("EndCameraAnimation");

        //pour le réseau : récupérer tout les players et boucler sur chacun
        GameObject.Find("ScriptGame").GetComponent<Game>().InitNetwork();

        //trouve le joueur
        GameObject endPlayer = GameObject.Find("player" + Lobby.login).gameObject;
        //le positionne au spawnpoint 
        //endPlayer.transform.position = GameObject.Find("SpawnPointPlayer1").transform.position;
        endPlayer.transform.localScale = new Vector3(0.15f, 0.15f, 0.15f);
        endPlayer.transform.eulerAngles = new Vector3(0f, 0f, 0f);
        //active l'animation de danse 
        endPlayer.GetComponent<Animator>().SetBool("isDancing", true);
        //désactive ses contrôles
        endPlayer.GetComponent<PlayerController>().enabled = false;
        endPlayer.GetComponent<PlayerInput>().enabled = false;
        //enlève l'arme (en déterminant si c'est l'archer ou le warrior par leurs chapeaux Hat/helmet)
        if (endPlayer.transform.Find("Hat") != null)
            Destroy(endPlayer.transform.Find("mixamorig:Hips/mixamorig:Spine/mixamorig:Spine1/mixamorig:Spine2/mixamorig:LeftShoulder/mixamorig:LeftArm/mixamorig:LeftForeArm/mixamorig:LeftHand/arc2").gameObject);
        else
            Destroy(endPlayer.transform.Find("mixamorig:Hips/mixamorig:Spine/mixamorig:Spine1/mixamorig:Spine2/mixamorig:RightShoulder/mixamorig:RightArm/mixamorig:RightForeArm/mixamorig:RightHand/epeeLongue").gameObject);
            Destroy(endPlayer.transform.Find("mixamorig:Hips/mixamorig:Spine/mixamorig:Spine1/mixamorig:Spine2/mixamorig:LeftShoulder/mixamorig:LeftArm/mixamorig:LeftForeArm/mixamorig:LeftHand/shield").gameObject);

        StartCoroutine(EndButton());

    }

    public IEnumerator EndButton()
    {
        
        button = screen.Find("SpectateButton");
        
        // Définit la fonction à appeler lorsqu'on clique sur le bouton
        button.GetComponent<Button>().onClick.AddListener(ButtonQuitOnClick);

        yield return new WaitForSeconds(7);
        button.gameObject.SetActive(true);
    }

    // Fonction qu'on appelle lorsqu'on clique sur le bouton
    public void ButtonQuitOnClick()
    {
        button.gameObject.SetActive(false);
        QuitGame();
    }

    // quitte la partie en changeant de scène
    public void QuitGame()
    {
        Game.LeaveGame();

        StopAllCoroutines();

        // suppression de tous les gameObject inutile du DontDestroyOnLoad
        Destroy(GameObject.Find("Canvas"));
        Destroy(GameObject.Find("CameraManager"));
        Destroy(GameObject.Find("MinimapCamera"));
        Destroy(GameObject.Find("player" + Lobby.login));
        Destroy(GameObject.Find("Main_Camera"));

        GameObject[] enemies = GameObject.FindGameObjectsWithTag("enemy");
        GameObject[] players = GameObject.FindGameObjectsWithTag("player");
        for (int i = 0; i < enemies.Length; i++) // supprimer tous les ennemies créé
        {
            Destroy(enemies[i]);
        }
        for (int i = 0; i < players.Length; i++) // supprimer tous les ennemies créé
        {
            Destroy(players[i]);
        }
    }

    IEnumerator UpdateStats()
    {
        // Créé un formulaire d'envoie spécifique pour être compris par la BDD
        GameObject endPlayer = GameObject.Find("player" + PlayerInfo.GetUserName()).gameObject;
        Inventory playerInventory = endPlayer.GetComponent<Inventory>();
        Debug.Log(playerInventory.coins);
        WWWForm form = new WWWForm();
        form.AddField("userName", PlayerInfo.GetUserName());
        form.AddField("goldCoins", playerInventory.coins);

        // On communique les informations
        using (UnityWebRequest webRequest = UnityWebRequest.Post("https://mai-projet-integrateur.u-strasbg.fr/vmProjetIntegrateurgrp6-0/update_player_stats.php", form))
        {
            webRequest.certificateHandler = new CertificateOverride();
            yield return webRequest.SendWebRequest();

            // La connexion au serveur a echoue
            if (webRequest.result == UnityWebRequest.Result.ConnectionError || webRequest.result == UnityWebRequest.Result.ProtocolError)
            {
                Debug.Log(webRequest.error);
            }
        }
    }
}
