using UnityEngine;
using UnityEngine.SceneManagement;
using TMPro;
using UnityWebSocket;
using Newtonsoft.Json;
using UnityEngine.UI;


public class ConnectionGame : MonoBehaviour
{
    // les boutons de l'ui
    [SerializeField] private Button buttonCreate;
    [SerializeField] private Button buttonJoin;
    [SerializeField] private Button buttonRetryNetwork;

    // les textes de l'ui (messages d'erreur et login)
    [SerializeField] private GameObject textError;
    [SerializeField] private TMP_Text textErrorText;

    // le champ de texte pour le room code
    [SerializeField] private TMP_InputField inputField;
    [SerializeField] private TMP_Text textRoomCode;


    //---------------------------------------------------------------------------------------------------------------------
    // ADRESSE DU SERVEUR (serveur Unistra que si connecté au réseau Unistra !)
    public static string serverMuonAddress = "ws://muon.tplinkdns.com:10003";
    public static string serverUnistraAddress = "ws://185.155.93.105:16003";

    public static string currentServer = "";
    //---------------------------------------------------------------------------------------------------------------------


    // pour retenir localement si on rejoint ou crée un lobby
    private bool createLobby = true;


    private void Start()
    {
        // par défaut on utilise mon serveur car il est accessible partout 
        if (currentServer == "")
            currentServer = serverMuonAddress;

        if (GameObject.Find("ScriptGame") != null)
            Destroy(GameObject.Find("ScriptGame"));

        GameObject[] lostPlayers = GameObject.FindGameObjectsWithTag("player");
        foreach (GameObject player in lostPlayers)
            Destroy(player);

        // les listeneurs pour les boutons d'ui
        buttonCreate.onClick.AddListener(CreateLobby);
        buttonJoin.onClick.AddListener(JoinLobby);
        buttonRetryNetwork.onClick.AddListener(ConnectionServer);

        // les listeneurs pour le champ du room code
        inputField.onValueChanged.AddListener(setUppercaseAndFormat);
        inputField.onEndEdit.AddListener(pressEnterInTextInput);

        // on se connecte au serveur
        Lobby.socket = new WebSocket(currentServer);

        ConnectionServer();

        // quand la connexion s'est ouverte correctement
        Lobby.socket.OnOpen += (sender, message) =>
        {
            Debug.Log("Connection open!");

            textErrorText.text = "";
            textError.SetActive(false);
        };

        // la fonctione appelée quand on reçoit un paquet du serveur
        Lobby.socket.OnMessage += FunctionOnMessage1;

        // quand la socket se ferme (problème si on est là !)
        Lobby.socket.OnClose += (sender, message) =>
        {
            Debug.Log("Socket closed!");

            // on en informe le joueur quand même
            textErrorText.text = "La connexion au serveur a échoué.";
            textError.SetActive(true);

            // et on affiche le bouton pour se reconnecter
            buttonRetryNetwork.gameObject.SetActive(true);
        };
    }

    public void ConnectionServer()
    {
        Debug.Log("Connection to " + currentServer);

        Lobby.socket.ConnectAsync();

        // écran de chargement du pauvre
        textErrorText.text = "Connexion au serveur en cours...";
        textError.SetActive(true);

        buttonRetryNetwork.gameObject.SetActive(false);
    }

    public void DeconnectionServer()
    {
        Debug.Log("Deconnection from " + currentServer);

        try { Lobby.socket.CloseAsync(); } 
        
        catch { }  
    }

    // à chaque paquet reçu du serveur
    public void FunctionOnMessage1(object sender, MessageEventArgs message)
    {
        if (message.IsText)
        {
            // on parse le paquet JSON reçu du serveur
            Lobby.infoRecvSimple = JsonConvert.DeserializeObject<PlayerDataSimple>(message.Data);

            // et on traite selon le code d'erreur reçu
            switch (int.Parse(Lobby.infoRecvSimple.errorCode))
            {
                // on a bien (créé et) rejoint le lobby
                case 0:
                    {
                        textErrorText.text = "";
                        textError.SetActive(false);

                        // on mets à jour les infos locales
                        Lobby.infoSendSimple.createLobby = "false";
                        Lobby.infoSendSimple.inLobby = "true";

                        // on récupère le numéro de joueur et le code de la salle
                        Lobby.playerNumber = int.Parse(Lobby.infoRecvSimple.playerNumber);
                        Lobby.infoSendSimple.playerNumber = Lobby.playerNumber.ToString();

                        // on mets encore à jour les infos locales
                        Lobby.roomCode = Lobby.infoRecvSimple.roomCode;
                        Lobby.infoSendSimple.roomCode = Lobby.roomCode;

                        Lobby.socket.OnMessage -= FunctionOnMessage1;

                        // et on se casse dans le lobby !
                        SceneManager.LoadScene("Lobby3D");

                        break;
                    }

                // on veut créer un lobby dans un lobby (ne devrait jamais arriver)
                case 11:
                    {
                        Debug.Log("Déjà présent dans le lobby");

                        // on informe le client comme on peut
                        textErrorText.text = "Une erreur est survenue";
                        textError.SetActive(true);

                        break;
                    }

                // le lobby qu'on veut joindre c'est du bullshit
                case 22:
                    {
                        Debug.Log("Code lobby non trouvé");

                        // on dit au client qu'il a tapé un mauvais code
                        textErrorText.text = "Le lobby que vous essayez de rejoindre n'existe pas\nVérifiez votre code, ou essayez de changer de serveur dans les paramètres";
                        textError.SetActive(true);

                        break;
                    }
                
                // il y a déjà 4 joueurs dans le lobby
                case 23:
                    {
                        Debug.Log("Lobby plein");

                        // on envoie chier gentillement le client
                        textErrorText.text = "Le lobby que vous essayez de joindre est plein.";
                        textError.SetActive(true);

                        break;
                    }

                // le joueur est déjà dans un lobby (plusieurs personne sur le même code)
                case 24:
                    {
                        Debug.Log("Joueur déjà présent dans un lobby (plusieurs personne sur le même compte !)");

                        // le serveur dépend de l'unicité des logins alors on laisse pas passer ça
                        textErrorText.text = "Vous êtes déjà dans un lobby sur un autre appareil.";
                        textError.SetActive(true);

                        break;
                    } 

                // la partie a déjà commencé
                case 25:
                    {
                        Debug.Log("Partie déjà commencée");

                        // on envoie cheh au client
                        textErrorText.text = "La partie dans ce lobby a déjà commencé.";
                        textError.SetActive(true);

                        break;
                    }
            }
        }
    }

    // appelé à chaque caractère tapé dans le champ de texte
    public void setUppercaseAndFormat(string newValue)
    {
        // on limite la taille à 6
        if (newValue.Length > 6)
        {
            newValue = newValue.Substring(0, 6);
        }

        // on garde que les lettres
        string filteredValue = "";
        foreach (char c in newValue)
        {
            if (char.IsLetter(c))
            {
                filteredValue += c;
            }
        }

        // et on met le tout en majuscules
        inputField.text = filteredValue.Trim().ToUpper();
        textRoomCode.text = filteredValue.Trim().ToUpper();
    }

    // quand on appuie sur entrée dans le champ de texte
    public void pressEnterInTextInput(string newValue)
    {
        createLobby = false;
        CreateOrJoinLobby();
    }

    // quand on ferme le jeu
    public void OnApplicationQuit()
    {
        DeconnectionServer();
    }

    // quand on appuie sur créer un lobby
    public void CreateLobby()
    {
        createLobby = true;
        CreateOrJoinLobby();
    }

    // quand on appuie sur joindre un lobby
    public void JoinLobby()
    {
        createLobby = false;
        CreateOrJoinLobby();
    }

    // on crée ou rejoins un lobby
    public void CreateOrJoinLobby()
    {
        Lobby.login = PlayerInfo.GetUserName();

        // on chope le code qui est saisi dans le champ
        Lobby.roomCode = textRoomCode.text;
        Lobby.roomCode = Lobby.roomCode.Remove(Lobby.roomCode.Length - 1);

        // si on veut rejoindre un lobby mais qu'aucun code n'est donné
        if (!createLobby && Lobby.roomCode == "")
        {
            // on informe le joueur qu'il n'est pas malin
            textErrorText.text = "Renseignez le code du lobby";
            textError.SetActive(true);
            return;
        }

        // on créer la structure de données pour le serveur
        Lobby.infoSendSimple = new PlayerDataSimple();

        // on se prépare à envoyer nos données au serveur
        Lobby.infoSendSimple.login = Lobby.login;
        Lobby.infoSendSimple.roomCode = (createLobby ? "" : Lobby.roomCode);
        Lobby.infoSendSimple.createLobby = (createLobby ? "true" : "false");
        Lobby.infoSendSimple.inLobby = "false";
        Lobby.infoSendSimple.isReady = "false";
        Lobby.infoSendSimple.errorCode = "0";
        Lobby.infoSendSimple.errorMsg = "";
        Lobby.infoSendSimple.currentScene = "Lobby3D";
        Lobby.infoSendSimple.animMoveForward = "false";
        Lobby.infoSendSimple.animMoveBackward = "false";
        Lobby.infoSendSimple.animMoveRight = "false";
        Lobby.infoSendSimple.animMoveLeft = "false";
        Lobby.infoSendSimple.animAttack = "false";
        Lobby.infoSendSimple.animSpeAtt = "false";
        Lobby.infoSendSimple.animRoll = "false";
        Lobby.infoSendSimple.animShield = "false";
        Lobby.infoSendSimple.hitPoint = "666";
        Lobby.infoSendSimple.type = "Warrior";
        Lobby.infoSendSimple.posX = "69";
        Lobby.infoSendSimple.posY = "69";
        Lobby.infoSendSimple.posZ = "69";
        Lobby.infoSendSimple.rotY = "0";

        // et enfin on envoie ces données
        Lobby.socket.SendAsync(JsonConvert.SerializeObject(Lobby.infoSendSimple));
    }
}