using UnityEngine;
using TMPro;
using UnityWebSocket;
using Newtonsoft.Json;
using UnityEngine.UI;
using UnityEngine.SceneManagement;
using System.Collections.Generic;
using UnityEngine.InputSystem;
using System.Collections;

public class Lobby : MonoBehaviour
{
    // les écrans dans le canvas
    [SerializeField] private GameObject playerSelectionDisplay;
    [SerializeField] private GameObject mainDisplay;

    // les boutons de l'ui
    [SerializeField] private Button buttonSelect;
    [SerializeField] private Button buttonStart;
    [SerializeField] private Button buttonReady;
    [SerializeField] private Button buttonBack;

    // les textes avec les noms de joueurs et status de l'ui
    [SerializeField] private TMP_Text[] textPlayerNameTextLobby;

    // le texte du code de la salle
    [SerializeField] private TMP_Text textRoomCodeText;

    // les différentes préfab des joueurs (à compléter en temps voulu)
    [SerializeField] private GameObject prefabWarrior;
    [SerializeField] private GameObject prefabArcher;


    // les données liées au réseau accessibles partout
    public static WebSocket socket;
    public static PlayerDataSimple infoSendSimple;
    public static PlayerDataSimple infoRecvSimple;
    public static PlayerDataOnline infoRecvOnline;

    // les données concernant la partie en cours pour le joueur local
    public static GameObject player;
    public static string login;
    public static bool isMainPlayer = false;
    public static string roomCode = "";
    public static int playerNumber = -1;
    public static int nbPlayers = 1;

    // les types possibles pour les joueurs (calcul du nombre de joueurs)
    public static List<string> playersType = new List<string>();


    private void Start()
    {
        playersType.Add("Warrior");
        playersType.Add("Archer");

        // si on relance une partie, il faut qu'on parte du premier floor
        ScriptGenerationDonjon.floor = 1;

        // on update le code de la salle avec celui obtenu du serveur
        textRoomCodeText.text = roomCode;

        // le bouton start sera dispo uniquement pour le maitre du jeu
        buttonStart.gameObject.SetActive(false);

        // les listeners des boutons de l'ui
        buttonSelect.onClick.AddListener(SelectCharacter);
        buttonStart.onClick.AddListener(StartGame);
        buttonReady.onClick.AddListener(ChangeStatus);
        buttonBack.onClick.AddListener(LeaveRoom);

        // la fonction qui sera appelée à chaque paquet reçu du serveur
        socket.OnMessage += FunctionOnMessage2;

        // fréquence des envois réseau : 30x par seconde là (0.033f)
        InvokeRepeating("SendDataToServer", 0, 0.033f);
    }

    public static void InvokeCharacter()
    {
        // on cherche le point de spawn en fonction de notre numéro de joueur
        GameObject startPosition = GameObject.Find("Spawn" + playerNumber);

        Vector3 currentPosition = startPosition.transform.position;

        if (player != null)
        {
            currentPosition = player.transform.position;
            Destroy(player);
        }

        // on charge le joueur local au point de spawn au chargement de la scène
        player = Instantiate(PlayerSelection.currentPrefab, currentPosition, Quaternion.Euler(0, 0, 0));
        player.name = "player" + login;

        infoSendSimple.type = PlayerSelection.currentPrefabName;
    }

    public void SelectCharacter()
    {
        playerSelectionDisplay.SetActive(true);
        mainDisplay.SetActive(false);

        if (infoSendSimple.isReady == "true")
            ChangeStatus();
    }

    // la fonction appelée à chaque paquet reçu du serveur
    public void FunctionOnMessage2(object sender, MessageEventArgs message)
    {
        if (message.IsText)
        {
            // on parse le JSON reçu par le serveur
            infoRecvOnline = JsonConvert.DeserializeObject<PlayerDataOnline>(message.Data);

            // on traite le paquet reçu selon le code erreur
            switch (int.Parse(infoRecvOnline.errorCode))
            {
                // le cas où tout va bien (quasiment toujours ce cas)
                case 0:
                    {
                        // si le maitre de jeu a changé de scène, on fait pareil
                        if (!isMainPlayer && SceneManager.GetActiveScene().name != infoRecvOnline.currentScene)
                        {
                            socket.OnMessage -= FunctionOnMessage2;
                            SceneManager.LoadScene(infoRecvOnline.currentScene);
                        }

                        // on mets à jour l'ui (noms et status)
                        UpdateStatusAndNames();

                        // on vérifie si on a récupéré le role du maitre de partie
                        if (infoRecvOnline.mainPlayer == login)
                            isMainPlayer = true;

                        // pour chacun des joueurs dans le lobby...
                        for (int i = 0; i < infoRecvOnline.login.Length; i++)
                        {
                            // on ignore son propre joueur (à voir pour la securité)
                            if (infoRecvOnline.login[i] == login) continue;

                            // on obtient les nouvelles postion et rotation
                            Vector3 newPosition = new Vector3(float.Parse(infoRecvOnline.posX[i], System.Globalization.CultureInfo.GetCultureInfo("fr-FR")), float.Parse(infoRecvOnline.posY[i], System.Globalization.CultureInfo.GetCultureInfo("fr-FR")), float.Parse(infoRecvOnline.posZ[i], System.Globalization.CultureInfo.GetCultureInfo("fr-FR")));
                            Quaternion newRotation = Quaternion.Euler(0, float.Parse(infoRecvOnline.rotY[i], System.Globalization.CultureInfo.GetCultureInfo("fr-FR")), 0);

                            // on chope le joueur allié concerné
                            GameObject allie = GameObject.Find("player" + infoRecvOnline.login[i]);

                            // si l'allié est déjà là mais qu'il a changé de classe on le défonce
                            if (allie != null && (allie.GetComponent<Warriors>() != null && infoRecvOnline.type[i] != "Warrior" || allie.GetComponent<Archers>() != null && infoRecvOnline.type[i] != "Archer"))
                                Destroy(allie);

                            // si on le trouve pas, c'est qu'il vient d'arriver, ou qu'on vient de le supprimer parce qu'il a changé de classe
                            if (allie == null)
                            {
                                // alors on le crée
                                allie = Instantiate((infoRecvOnline.type[i] == "Warrior" ? prefabWarrior : prefabArcher), newPosition, newRotation);
                                allie.name = "player" + infoRecvOnline.login[i];

                                // on met la vitesse de l'animation de marche à 1
                                allie.GetComponent<Animator>().SetFloat("SpeedMultiplier", 1);

                                // et on désactive les objets qui ne doivent pas interférer (important !)
                                allie.GetComponent<PlayerController>().enabled = false;
                                allie.GetComponent<PlayerCameraFocus>().enabled = false;
                                allie.GetComponent<PlayerInput>().enabled = false;

                                // on mets à jour le nombre de joueurs
                                nbPlayers = 0;
                                for (int k = 0; k < infoRecvOnline.login.Length; k++)
                                {
                                    if (playersType.IndexOf(infoRecvOnline.type[k]) != -1)
                                        nbPlayers++;
                                }
                            }

                            // enfin on applique les nouvelles position et rotation
                            allie.transform.position = newPosition;
                            allie.transform.rotation = newRotation;

                            // gestion des PV
                            allie.GetComponent<Players>().stats.currentHp = int.Parse(infoRecvOnline.hitPoint[i]);
                            if (infoRecvOnline.animMoveForward[i] == "true")
                                allie.GetComponent<Animator>().SetBool("MovingForward", true);
                            else
                                allie.GetComponent<Animator>().SetBool("MovingForward", false);
                            if (infoRecvOnline.animMoveBackward[i] == "true")
                                allie.GetComponent<Animator>().SetBool("MovingBackward", true);
                            else
                                allie.GetComponent<Animator>().SetBool("MovingBackward", false);
                            if (infoRecvOnline.animMoveRight[i] == "true")
                                allie.GetComponent<Animator>().SetBool("MovingRight", true);
                            else
                                allie.GetComponent<Animator>().SetBool("MovingRight", false);

                            if (infoRecvOnline.animMoveLeft[i] == "true")
                                allie.GetComponent<Animator>().SetBool("MovingLeft", true);
                            else
                                allie.GetComponent<Animator>().SetBool("MovingLeft", false);

                            if (infoRecvOnline.animAttack[i] == "true")
                            {
                                if (infoRecvOnline.type[i] == "Warrior")
                                    StartCoroutine(allie.GetComponent<Warriors>().Attack(infoRecvOnline.login[i]));
                                else if (infoRecvOnline.type[i] == "Archer")
                                    StartCoroutine(allie.GetComponent<Archers>().Attack(infoRecvOnline.login[i]));
                            }

                            if (infoRecvOnline.animSpeAtt[i] == "true")
                            {
                                if (infoRecvOnline.type[i] == "Warrior")
                                    StartCoroutine(allie.GetComponent<Warriors>().SecondAttack(infoRecvOnline.login[i]));
                                else if (infoRecvOnline.type[i] == "Archer")
                                    StartCoroutine(allie.GetComponent<Archers>().SecondAttack(infoRecvOnline.login[i]));
                            }

                            if (infoRecvOnline.type[i] == "Archer" && infoRecvOnline.animRoll[i] == "true")
                            {
                                StartCoroutine(allie.GetComponent<Archers>().SpecialMovement(infoRecvOnline.login[i], Vector2.zero));
                            }

                            if (infoRecvOnline.type[i] == "Warrior" && infoRecvOnline.animShield[i] == "true")
                            {
                                allie.GetComponent<Warriors>().StartBlock();
                            }
                            if (infoRecvOnline.type[i] == "Warrior" && infoRecvOnline.animShield[i] == "false")
                            {
                                allie.GetComponent<Warriors>().EndBlock();
                            }
                        }

                        break;
                    }

                // on reçoit la confirmation qu'on a bien quitté le lobby
                case 24:
                    {
                        // on reset le numéro de joueur et le code de room
                        playerNumber = -1;
                        roomCode = "";

                        socket.OnMessage -= FunctionOnMessage2;

                        CancelInvoke("SendDataToServer");

                        // si jamais on était chef de partie, bah on l'est plus
                        isMainPlayer = false;

                        // et on retourne au menu principal
                        SceneManager.LoadScene("Main Menu");

                        break;
                    }

                // l'un des joueurs du lobby s'est cassée (volontairement ou non)
                case 40:
                    {
                        // on mets à jour l'ui (noms et status)
                        UpdateStatusAndNames();

                        // on mets à jour le nombre de joueurs et récupère son potentiel nouveau numéro de joueur
                        nbPlayers = 0;
                        for (int k = 0; k < infoRecvOnline.login.Length; k++)
                        {
                            if (playersType.IndexOf(infoRecvOnline.type[k]) != -1)
                                nbPlayers++;

                            if (infoRecvOnline.login[k] == login)
                                playerNumber = nbPlayers;
                        }

                        // on chope tous les joueurs alliés sur la scène
                        List<GameObject> allies = new List<GameObject>(GameObject.FindGameObjectsWithTag("player"));

                        // on enlève de la liste ceux dont on a encore reçu des infos du serveur
                        for (int i = 0; i < infoRecvOnline.login.Length; i++)
                        {
                            if (GameObject.Find("player" + infoRecvOnline.login[i]) != null)
                                allies.Remove(GameObject.Find("player" + infoRecvOnline.login[i]));
                        }

                        // comme ça on supprime que ceux qui sont partis !
                        foreach (GameObject allie in allies)
                        {
                            Destroy(allie);
                        }

                        break;
                    }
            }
        }
    }

    // quand on ferme le jeu
    public void OnApplicationQuit()
    {
        try
        {
            // on ferme la websocket car elle se ferme pas toute seule en fait c'est nul
            socket.CloseAsync();
        }
        catch {}
    }

    // quand on appuie sur le bouton qui change le status
    public void ChangeStatus()
    {
        // on inverse son status
        infoSendSimple.isReady = infoSendSimple.isReady == "true" ? "false" : "true";

        // et on l'envoi
        socket.SendAsync(JsonConvert.SerializeObject(infoSendSimple));
    }

    // quand on quitte le lobby
    public void LeaveRoom()
    {
        // on se prépare à dire au serveur qu'on se casse
        infoSendSimple.roomCode = "";
        infoSendSimple.inLobby = "false";

        // puis on lui dit
        socket.SendAsync(JsonConvert.SerializeObject(infoSendSimple));
    }

    // l'envoi des données du joueur local au serveur (selon la fréquence définit plus haut)
    public void SendDataToServer()
    {
        // on récupère la scène actuelle
        infoSendSimple.currentScene = SceneManager.GetActiveScene().name;

        // on se prépare à envoyer ses points de vie 
        infoSendSimple.hitPoint = player.GetComponent<Players>().stats.currentHp.ToString();

        // on se prépare à envoier sa position et rotation
        infoSendSimple.posX = player.transform.position.x.ToString().Replace(".", ",");
        infoSendSimple.posY = player.transform.position.y.ToString().Replace(".", ",");
        infoSendSimple.posZ = player.transform.position.z.ToString().Replace(".", ",");
        infoSendSimple.rotY = player.transform.localEulerAngles.y.ToString().Replace(".", ",");

        // envoie des données au serveur qui les redonnera aux autres joueurs
        socket.SendAsync(JsonConvert.SerializeObject(infoSendSimple));

        // on enlève les animations qui ne doivent être jouées qu'une seule fois (peut-être ?)
        // ça me semble important mais je sais pas comment ça marche
        infoSendSimple.animAttack = "false";
        infoSendSimple.animSpeAtt = "false";
        infoSendSimple.animRoll = "false";
    }

    // on mets à jour l'ui (noms et status)
    private void UpdateStatusAndNames()
    {
        int nbReady = 0;

        // d'abord on efface tout
        for (int i = 0; i < 4; i++)
        {
            textPlayerNameTextLobby[i].text = "";
            textPlayerNameTextLobby[i].color = Color.white;
        }

        // puis on réaffiche selon les données reçues
        for (int i = 0; i < infoRecvOnline.login.Length; i++)
        {
            textPlayerNameTextLobby[i].text = infoRecvOnline.login[i] + (infoRecvOnline.status[i] == "true" ? "  <color=green>Prêt</color>" : "  <color=red>Pas prêt</color>");

            // le texte en rouge pour le maitre du jeu
            // correspond à la couleur violent foncée des boutons
            if (infoRecvOnline.login[i] == infoRecvOnline.mainPlayer)
                textPlayerNameTextLobby[i].color = new Color((float)0.11372549019, (float)0.05490196078, (float)0.1725490196);

            // et on compte si le joueur est prêt
            if (infoRecvOnline.status[i] == "true") nbReady++;
        }

        // si on est le maitre du jeu ET tout le monde est prêt, alors le bouton start s'affiche
        if (nbReady == infoRecvOnline.login.Length && login == infoRecvOnline.mainPlayer)
            buttonStart.gameObject.SetActive(true);
        else
            buttonStart.gameObject.SetActive(false);
    }

    // quand le maitre du jeu (seulement lui) appuie sur start
    private void StartGame()
    {
        socket.OnMessage -= FunctionOnMessage2;

        // on se prépare à dire au serveur qu'on change de scène
        infoSendSimple.currentScene = "Floor1";

        // puis on lui dit
        socket.SendAsync(JsonConvert.SerializeObject(infoSendSimple));

        // mais on oublie pas de changer la scène soi-même
        SceneManager.LoadScene("Floor1");
    }
}
