using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;


public class PlayerController : MonoBehaviour
{
    Rigidbody rb;
    Vector3 moveVector;
    public Players player;

    public Animator animator;

    public static bool isInLobby = true;

    public bool enableLookAtRotation = false;

    public Vector2 direction;

    // SFX
    public AudioSource audioSrc;
    public List<AudioClip> sfxCoinPickup;
    public List<AudioClip> sfxPotionPickup;
    public List<AudioClip> sfxKeyPickup;
    public AudioClip sfxOpenUnlockedChest;
    public AudioClip sfxOpenLockWithKey;
    public List<AudioClip> sfxChestCoins;
    public AudioClip sfxHeal;
    public AudioClip sfxPowerUp;

    // Start is called before the first frame update
    void Start()
    {
        animator = player.animator;

        rb = GetComponent<Rigidbody>();
        StartCoroutine(EnableLookAtRotation());
    }

    IEnumerator EnableLookAtRotation()
    {
        yield return new WaitForSeconds(.1f);
        enableLookAtRotation = true;
    }

    //Implémentation du ramassage d'objet (Si y'a le temps faudrait refaire ça pour que chaque objet hérite d'une classe Objet abstraite pour éviter la liste)
    void OnTriggerEnter(Collider other)
    {
        if(player.gameObject.name == ("player" + Lobby.login))
        {
            if (other.tag == "object")
            {
                if (other.name == "Piece(Clone)")
                {
                    player.getInventory().addCoins();
                    Destroy(other.gameObject);
                    audioSrc.PlayOneShot(sfxCoinPickup[ScriptGenerationDonjon.randomPersonal.Next(0, sfxCoinPickup.Count)], 0.5f);
                }
                else if (other.name == "HealthPotion(Clone)")
                {
                    player.getInventory().addHealthPotion();
                    Destroy(other.gameObject);
                    audioSrc.PlayOneShot(sfxPotionPickup[ScriptGenerationDonjon.randomPersonal.Next(0, sfxPotionPickup.Count)], 0.5f);
                }
                else if (other.name == "Key(Clone)")
                {
                    player.getInventory().addKey();
                    Destroy(other.gameObject);
                    audioSrc.PlayOneShot(sfxKeyPickup[ScriptGenerationDonjon.randomPersonal.Next(0, sfxKeyPickup.Count)], 0.5f);
                }
            }
        }
        else if (other.tag == "fog")
        {
            
        }

    }

    void FixedUpdate()
    {
        player.animator.SetFloat("SpeedMultiplier", player.stats.speed/10);

        if(player.stats.currentHp <=0)
            return;

        if (!animator.GetBool("IsTriggered") && player.finishedInit)
        {
            rb.velocity = moveVector * player.stats.speed;
        }
        else
        {
            rb.velocity = Vector3.zero;
        }
        //oriente le joueur vers la souris
        RaycastHit hit;

        int layermask = 1 << 6; // TODO
        layermask = ~layermask;
        Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
        if (Physics.Raycast(ray, out hit, 9999, layermask) && enableLookAtRotation) // 6 pour éviter les ennemis
            transform.LookAt(new Vector3(hit.point.x, transform.position.y, hit.point.z));
    }

    public void OnMove(InputAction.CallbackContext context)
    {

        direction = context.ReadValue<Vector2>();
        Vector2 playerRot = new Vector2(transform.forward.x, transform.forward.z);
        float angle = Vector2.Angle(new Vector2(transform.forward.x, transform.forward.z), Vector2.right);
        // permet de changer les animations selon le regard du personnage
        if (angle < 45 || angle > 135) // on regarde vers la droite ou vers la gauche
        {
            animator.SetBool("MovingForward", direction.x * playerRot.x > 0);
            Lobby.infoSendSimple.animMoveForward = (direction.x * playerRot.x > 0 ? "true" : "false");
            animator.SetBool("MovingBackward", direction.x * playerRot.x < 0);
            Lobby.infoSendSimple.animMoveBackward = (direction.x * playerRot.x < 0 ? "true" : "false");
            animator.SetBool("MovingRight", direction.y * playerRot.x < 0);
            Lobby.infoSendSimple.animMoveRight = (direction.y * playerRot.x < 0 ? "true" : "false");
            animator.SetBool("MovingLeft", direction.y * playerRot.x > 0);
            Lobby.infoSendSimple.animMoveLeft = (direction.y * playerRot.x > 0 ? "true" : "false");
        }
        else if (angle < 135) // on regarde vers le haut ou vers le bas
        {
            animator.SetBool("MovingForward", direction.y * playerRot.y > 0);
            Lobby.infoSendSimple.animMoveForward = (direction.y * playerRot.y > 0 ? "true" : "false");
            animator.SetBool("MovingBackward", direction.y * playerRot.y < 0);
            Lobby.infoSendSimple.animMoveBackward = (direction.y * playerRot.y < 0 ? "true" : "false");
            animator.SetBool("MovingRight", direction.x * playerRot.y > 0);
            Lobby.infoSendSimple.animMoveRight = (direction.x * playerRot.y > 0 ? "true" : "false");
            animator.SetBool("MovingLeft", direction.x * playerRot.y < 0);
            Lobby.infoSendSimple.animMoveLeft = (direction.x * playerRot.y < 0 ? "true" : "false");
        }
        else // on désactive les animations si on fait rien
        {
            animator.SetBool("MovingForward", false);
            Lobby.infoSendSimple.animMoveForward = "false";
            animator.SetBool("MovingBackward", false);
            Lobby.infoSendSimple.animMoveBackward = "false";
            animator.SetBool("MovingRight", false);
            Lobby.infoSendSimple.animMoveRight = "false";
            animator.SetBool("MovingLeft", false);
            Lobby.infoSendSimple.animMoveLeft = "false";
        }
        // vecteur qui sert au déplacement du personnage
        moveVector = new Vector3(direction.x, 0, direction.y);
    }

    public void OnAttack(InputAction.CallbackContext context)
    {
        if (context.started && player.stats.currentHp > 0)
        {
            StartCoroutine(player.Attack(Lobby.login));
        }
    }

    public void OnSpecialMovement(InputAction.CallbackContext context)
    {
        if(gameObject.GetComponent<Warriors>() != null)
        {
            if (context.performed)
            {
                gameObject.GetComponent<Warriors>().StartBlock();
                Lobby.infoSendSimple.animShield = "true";
            }
            else if (context.canceled)
            {
                gameObject.GetComponent<Warriors>().EndBlock();
                Lobby.infoSendSimple.animShield = "false";
            }
        }
        else if (context.started && player.stats.currentHp > 0)
            StartCoroutine(gameObject.GetComponent<Archers>().SpecialMovement(Lobby.login, direction));
    }

    public void OnSecondAttack(InputAction.CallbackContext context)
    {
        if (context.started)
        {
            StartCoroutine(player.SecondAttack(Lobby.login));
            if(!isInLobby)
                GameObject.FindObjectOfType<SecondAttackCooldown>().StartCooldown();
        }
    }

    //Méthode d'interaction
    public void OnMerchant(InputAction.CallbackContext context)
    {
        if (context.started)
        {
            RaycastHit hit;

            //On s'intéresse uniquement aux objets de layer chest et merchant
            LayerMask layermask = LayerMask.GetMask("chest");
            LayerMask layerMaskMerchant = LayerMask.GetMask("merchant");

            //On lance un rayon pour chercher les objets de type chest qui sont soit les coffres soit l'échelle soit les powerUps
            Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
            if (Physics.Raycast(ray, out hit, 25, layermask.value))
            {
                if (hit.transform.gameObject.name == "Chest(Clone)")
                {
                    hit.transform.gameObject.GetComponent<Animator>().SetTrigger("OpeningChest");
                    audioSrc.PlayOneShot(sfxOpenUnlockedChest, 0.5f);
                    StartCoroutine(hit.transform.gameObject.GetComponent<ScriptChest>().SpawnLootOpenChest());
                    audioSrc.PlayOneShot(sfxChestCoins[ScriptGenerationDonjon.randomPersonal.Next(0, sfxChestCoins.Count)], 0.5f);
                }
                else if(hit.transform.gameObject.name == "ClosedChest(Clone)")
                {
                    audioSrc.PlayOneShot(sfxOpenLockWithKey, 0.5f);

                    //Fait spawn le contenu du coffre uniquement si le joueur a une clé
                    if (player.getInventory().RemoveKey())
                    {
                        hit.transform.gameObject.GetComponent<Animator>().SetTrigger("OpeningChest");
                        audioSrc.PlayOneShot(sfxOpenUnlockedChest, 0.5f);
                        StartCoroutine(hit.transform.gameObject.GetComponent<ScriptChest>().SpawnLootClosedChest());
                        audioSrc.PlayOneShot(sfxChestCoins[ScriptGenerationDonjon.randomPersonal.Next(0, sfxChestCoins.Count)], 0.5f);
                    }
                }
                else if(hit.transform.gameObject.name == "ladderPrefab 1")
                {
                    if (Lobby.isMainPlayer)
                        GameObject.FindObjectOfType<ScriptGenerationDonjon>().EventChangeFloor();
                }
                else if (hit.transform.gameObject.name.Contains("PowerUp"))
                {
                    hit.transform.gameObject.GetComponent<PowerUp>().Activate(player);
                    audioSrc.PlayOneShot(sfxPowerUp, 0.1f);
                }
                
            }
            //si le rayon n'a pas découvert d'objet chest alors lance un rayon pour trouver un marchand
            else if(Physics.Raycast(ray, out hit, 25, layerMaskMerchant.value))
            {
                hit.transform.gameObject.GetComponent<MerchantItem>().ActivateMerchantItem(gameObject);
            }
        }

    }

    public void OnHeal(InputAction.CallbackContext context)
    {
        if(context.started)
            if(player.Healing(500))
                audioSrc.PlayOneShot(sfxHeal, 0.1f);
    }

    public void TakeDamage(int damage)
    {
        player.TakeDamage(player.stats.clickDmgAbility, player.stats.power);
    }
}
