using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using TMPro;
using UnityEditor;
using System;


public abstract class Players : MonoBehaviour
{
    public static bool localPlayer;

    public Animator animator;
    public Transform attackPoint;
    public LayerMask enemyLayers;

    public float secondCD;
    public bool attackAvailable;
    public bool secondAttackAvailable;
    public event Action onTakeDamage;

    public bool canTakeDamage = true;

    public PlayersStatistics stats;
    public GameObject healthBar;

    public bool inEndGame = false;
    public Transform prefabSpawnPoint;
    public GameObject explosionPrefab;

    public GameObject healPrefab;
    // Start is called before the first frame update

    public bool finishedInit = false;

    protected Inventory inventory;

    // Event pour informer de la mort du personnage
    public event Action onDeath; 

    void Awake()
    {
        // DontDestroyOnLoad(gameObject);
        PlayersInit();
        if(gameObject.GetComponent<Inventory>() == null)
            gameObject.AddComponent<Inventory>();
    }

    public Inventory getInventory()
    {
        return GetComponent<Inventory>();
    }

    /*
     * Initialise les statistiques du joueur.
     */
    public abstract void PlayersInit();

    /*
     * Déclenche l'attaque du joueur.
     * Le type IEnmurator me permet d'ajouter au sein de la fonction, un petit délai permettant de
     * rendre les intéractions plus réalistes.
     */
    public abstract IEnumerator Attack(string login);

    public abstract IEnumerator SecondAttack(string login);

    public abstract IEnumerator SpecialMovement(string login, Vector2 direction);

    public abstract IEnumerator StartCoolDown(float coolDown, int idAttack);



    /*
     * Fonction qui fait perdre des points de vie au personnage. 
     */
    public void TakeDamage(int damageAbility, float damageAttack)
    {
        if (damageAbility - stats.defense > 0 && canTakeDamage)
        {
            stats.currentHp -= (int)((damageAttack / 100.0) * (damageAbility - stats.defense));
            onTakeDamage?.Invoke();
            animator.SetTrigger("GotHit");
        }
        healthBar.GetComponent<Slider>().value = stats.currentHp;
        Instantiate(explosionPrefab, prefabSpawnPoint.position, prefabSpawnPoint.rotation);
        if (stats.currentHp <= 0)
        {
            Die();
        }
    }

    /*
     * Fonction qui fait mourir le personnage.
     */
    public void Die()
    {
        //Lobby.nbPlayers--;

        onDeath?.Invoke();
        // Déclenche l'animation de mort
        animator.SetBool("IsDead", true);
        // Désactiver TOUS les collider car les ennemis utilisent ça pour taper
        Destroy(this.gameObject.GetComponent<BoxCollider>());
        Destroy(this.gameObject.GetComponent<CapsuleCollider>());
        Destroy(this.gameObject.GetComponent<Rigidbody>());
    }
    public bool Healing(int heal)
    {
        if(getInventory().usePotion())
        {
            stats.currentHp = (stats.currentHp + heal) > stats.maxHp? stats.maxHp: stats.currentHp + heal;
            if (GameObject.FindObjectOfType<HealthBar>() != null)
                GameObject.FindObjectOfType<HealthBar>().OnDamageTaken();     
            Instantiate(healPrefab, transform);
            return true;
        }
        return false;

    }

    void Update()
    {
        if(!inEndGame && transform.position.y > 0.6)
            transform.position = new Vector3(transform.position.x ,0.5f,transform.position.z);
        else if(!inEndGame && transform.position.y < -0.6)
            transform.position = new Vector3(transform.position.x ,0.5f,transform.position.z);

    }

    public string GetClassName()
    {
        return this.GetType().Name;
    }
}
