using System.Collections;
using System.Collections.Generic;
using TMPro;
using Unity.VisualScripting;
using UnityEngine;
using UnityEngine.InputSystem;
using UnityEngine.UI;

public class Archers : Players
{
    // Start is called before the first frame update
    [SerializeField] private GameObject arrowPrefab;
    public GameObject[] weapons;

    public AudioSource audioSrc;

    private bool canRoll = true;
    public List<AudioClip> sfxBow;
    public List<AudioClip> sfxArrow;

    // void Start()
    // {
    //     PlayersInit();
    // }

    /*
     * Initialise les statistiques du joueur.
     */
    public override void PlayersInit()
    {
        stats = ScriptableObject.CreateInstance<ArchersStatistics>();
        stats.init();
        healthBar.GetComponentInChildren<TextMeshProUGUI>().text =
            stats.pseudo;
        healthBar.GetComponent<Slider>().maxValue = stats.maxHp;
        healthBar.GetComponent<Slider>().value = stats.currentHp;
        attackAvailable = true;
        secondAttackAvailable = true;
        finishedInit = true;
    }

    
    
    /*
     * Déclenche l'attaque du joueur.
     * Le type IEnmurator me permet d'ajouter au sein de la fonction, un petit délai permettant de
     * rendre les intéractions plus réalistes.
     */
    public override IEnumerator Attack(string login)
    {
        if (!attackAvailable)
        {
            yield break;
        }
        // Déclenche l'animation d'attaque
        animator.SetTrigger("AutoAttacking");

        if (Lobby.login == login)
            Lobby.infoSendSimple.animAttack = "true";

        StartCoroutine(StartCoolDown(1f/stats.attackSpeed, 1));
        // Infliger des dégâts aux ennemis au bout de .2f s.
        yield return new WaitForSeconds(.2f);
        var arrow = Instantiate(arrowPrefab, prefabSpawnPoint.position, prefabSpawnPoint.rotation);
        // Ignorer les collisions avec certains objets
        IgnoreCollisionArrow(arrow);
        if (!Lobby.isMainPlayer)
            arrow.GetComponent<ArrowScript>().damageEnabled = false;

        //ignore la collision avec l'archer
        Physics.IgnoreCollision(arrow.GetComponent<Collider>(), GetComponent<Collider>());
        //ignore la colision avec les autres flèches
        weapons = GameObject.FindGameObjectsWithTag("weapon");
        foreach (GameObject weapon in weapons)
        {
            Physics.IgnoreCollision(weapon.GetComponent<Collider>(), arrow.GetComponent<Collider>());
        }
        
        arrow.GetComponent<ArrowScript>().power = stats.power;
        arrow.GetComponent<ArrowScript>().damages = stats.clickDmgAbility;
        arrow.GetComponent<Rigidbody>().velocity = ((ArchersStatistics)stats).arrowSpeed * prefabSpawnPoint.forward;
        audioSrc.PlayOneShot(sfxArrow[ScriptGenerationDonjon.randomPersonal.Next(0, sfxArrow.Count)], 0.1f);
    }
    

    public override IEnumerator SpecialMovement(string login, Vector2 direction)
    {
        if(canRoll)
        {
            //GetComponent<PlayerController>().enableLookAtRotation = false;
            canRoll = false;
            canTakeDamage = false;
            animator.SetTrigger("SpaceMove");
            
            if (Lobby.login == login)
                Lobby.infoSendSimple.animRoll = "true";
            
            
            yield return new WaitForSeconds(0.4f);
            canTakeDamage = true;
            yield return new WaitForSeconds(0.8f);
            canRoll = true;
            //GetComponent<PlayerController>().enableLookAtRotation = true;
        }

    }

    public override IEnumerator SecondAttack(string login)
    {
        if (!secondAttackAvailable)
        {
            yield break;
        }

        if (Lobby.login == login)
            Lobby.infoSendSimple.animSpeAtt = "true";

        // Cette ligne doit être placée immédiatement ici pour empêcher le joueur de spammer l'attaque durant l'animation
        StartCoroutine(StartCoolDown(stats.secondAttackCd, 2));
        // position / rotation à l'instant t du clic
        var tempPosition = prefabSpawnPoint.position;
        var tempForward = prefabSpawnPoint.forward;
        var tempRotation = prefabSpawnPoint.rotation;
        // Déclenche l'animation d'attaque
        animator.SetTrigger("AutoAttacking");

        // Lancer le projectile .2f secondes après le clic
        
        yield return new WaitForSeconds(.2f);
        
        for (int i = 0; i < ((ArchersStatistics)stats).nbArrows; i++)
        {
            yield return new WaitForSeconds(.1f);
            var arrow = Instantiate(arrowPrefab, tempPosition, tempRotation);
            // Ignorer les collisions avec certains objets
            IgnoreCollisionArrow(arrow);

            if (!Lobby.isMainPlayer)
                arrow.GetComponent<ArrowScript>().damageEnabled = false;

            arrow.GetComponent<ArrowScript>().power = stats.power;
            arrow.GetComponent<ArrowScript>().damages = stats.clickDmgAbility;
            arrow.GetComponent<Rigidbody>().velocity = ((ArchersStatistics)stats).arrowSpeed * tempForward;
            audioSrc.PlayOneShot(sfxArrow[ScriptGenerationDonjon.randomPersonal.Next(0, sfxArrow.Count)], 0.1f);
        }

        
    }

    public void IgnoreCollisionArrow(GameObject arrow)
    {
        //ignore la collision avec l'archer
        Physics.IgnoreCollision(arrow.GetComponent<Collider>(), GetComponent<Collider>());
        //ignore la colision avec les autres alliés
        GameObject[] players = GameObject.FindGameObjectsWithTag("player");
        foreach (GameObject player in players)
        {
            Physics.IgnoreCollision(player.GetComponent<Collider>(), arrow.GetComponent<Collider>());
        }
        //ignore la colision avec les autres flèches
        weapons = GameObject.FindGameObjectsWithTag("weapon");
        foreach (GameObject weapon in weapons)
        {
            Physics.IgnoreCollision(weapon.GetComponent<Collider>(), arrow.GetComponent<Collider>());
        }
    }


    public override IEnumerator StartCoolDown(float coolDown, int idAttack)
    {
        if (idAttack == 1)
        {
            attackAvailable = false;
            yield return new WaitForSeconds(coolDown);
            attackAvailable = true;
        }
        else if (idAttack == 2)
        {
            secondAttackAvailable = false;
            yield return new WaitForSeconds(coolDown);
            Debug.Log("spell available");
            secondAttackAvailable = true;
        }
    }
}
