using System.Collections;
using System.Collections.Generic;
using TMPro;
using Unity.VisualScripting;
using UnityEngine;
using UnityEngine.InputSystem;
using UnityEngine.UI;

public class Warriors : Players
{
    public List<string> animations;
    private int nbAttacks = 0;

    public AudioSource audioSrc;
    public List<AudioClip> sfxSwordSwing;
    public List<AudioClip> sfxSwordHit;
    public GameObject auraPrefab;
    public override void PlayersInit()
    {
        stats = ScriptableObject.CreateInstance<WarriorsStatistics>();
        stats.init();
        healthBar.GetComponentInChildren<TextMeshProUGUI>().text =
            stats.pseudo;
        healthBar.GetComponent<Slider>().maxValue = stats.maxHp;
        healthBar.GetComponent<Slider>().value = stats.currentHp;
        attackAvailable = true;
        secondAttackAvailable = true;
        animations.Add("AutoAttacking1");
        animations.Add("AutoAttacking2");
        finishedInit = true;
    }

    // Procédure permettant de repositionner la position du player local sur un vector3 passé en argument
    public void SetPlayerPosition(Vector3 newPosition)
    {
        gameObject.transform.position = newPosition;
    }

    void FixedUpdate()
    {
        healthBar.GetComponentInChildren<TextMeshProUGUI>().text =
            stats.pseudo;
    }

    
    public override IEnumerator SpecialMovement(string login, Vector2 direction)
    {
        yield return null;
    }
    public void StartBlock()
    {
        canTakeDamage = false;
        animator.SetBool("Blocking", true);
        animator.SetTrigger("SpaceMove");
    }

    public void EndBlock()
    {
        canTakeDamage = true;
        animator.SetBool("Blocking", false);
    }


    /*
     * Déclenche l'attaque du joueur.
     * Le type IEnmurator me permet d'ajouter au sein de la fonction, un petit délai permettant de
     * rendre les intéractions plus réalistes.
     */
    public override IEnumerator Attack(string login)
    {
        if (!attackAvailable)
        {
            yield break;
        }

        canTakeDamage = true;
        // Déclenche l'animation d'attaque
        int nbAnimations = animations.Count;
        animator.SetTrigger(animations[nbAttacks++ % nbAnimations]);

        if (Lobby.login == login)
        {
            Lobby.infoSendSimple.animAttack = "true";
            Lobby.infoSendSimple.animShield = "false";
        }

        StartCoroutine(StartCoolDown(1f / stats.attackSpeed, 1));

        // Détecte les ennemis à portée d'attaque
        Collider[] hitEnemies = Physics.OverlapSphere(attackPoint.position, stats.attackRange, enemyLayers);
        // Infliger des dégâts aux ennemis au bout de .2f s.
        yield return new WaitForSeconds(.2f);
        foreach (Collider enemy in hitEnemies)
        {

            if (enemy.tag == "enemy")
            {

                if(enemy.gameObject.GetComponent<Enemy>() == null)
                {
                    enemy.gameObject.GetComponentInParent<Enemy>().TakeDamage((Lobby.isMainPlayer ? stats.clickDmgAbility : 0), (Lobby.isMainPlayer ? stats.power : 0));
                }
                else
                {
                    enemy.gameObject.GetComponent<Enemy>().TakeDamage((Lobby.isMainPlayer ? stats.clickDmgAbility : 0), (Lobby.isMainPlayer ? stats.power : 0));
                }
                KnockBacking(enemy);
            }
            audioSrc.PlayOneShot(sfxSwordHit[ScriptGenerationDonjon.randomPersonal.Next(0, sfxSwordHit.Count)], 0.01f);
        }
        audioSrc.PlayOneShot(sfxSwordSwing[ScriptGenerationDonjon.randomPersonal.Next(0, sfxSwordSwing.Count)], 0.04f);
    }

    private void KnockBacking(Collider enemy)
    {
        Rigidbody enemyRB = enemy.GetComponent<Rigidbody>();
         if(enemyRB == null)
        {
            enemyRB = enemy.GetComponentInParent<Rigidbody>();
        }
        enemyRB.isKinematic = false;
        Vector3 difference = enemyRB.transform.position - transform.position;
        difference = difference.normalized * stats.thrust;
        enemyRB.AddForce(difference, ForceMode.Impulse);
        StartCoroutine(KnockCo(enemyRB));
    }

    private IEnumerator KnockCo(Rigidbody enemy)
    {
        if (enemy != null)
        {
            yield return new WaitForSeconds(stats.knockTime);
            enemy.velocity = Vector3.zero;
            enemy.isKinematic = true;
        }
    }

public override IEnumerator SecondAttack(string login)
    {
        if (!secondAttackAvailable)
        {
            yield break;
        }
        animator.SetTrigger("SpecialAttack");

        if (Lobby.login == login)
            Lobby.infoSendSimple.animSpeAtt = "true";

        StartCoroutine(StartCoolDown(stats.secondAttackCd, 2));
        Instantiate(auraPrefab, prefabSpawnPoint);
        float speedTemp = stats.speed;
        float powerTemp = stats.power;
        int defTemp = stats.defense;

        // Augmentation temporaires de stats
        stats.speed *= 2f;
        stats.power *= 1.30f;
        stats.defense *= stats.defense * 100 / 135;
        // durant 10s
        yield return new WaitForSeconds(10f);
        
        // remise à l'état normal
        stats.speed = speedTemp;
        stats.power = powerTemp;
        stats.defense = defTemp;

    }

    public override IEnumerator StartCoolDown(float coolDown, int idAttack)
    {
        if (idAttack == 1)
        {
            attackAvailable = false;
            yield return new WaitForSeconds(coolDown);
            attackAvailable = true;
        }
        else if (idAttack == 2)
        {
            secondAttackAvailable = false;
            yield return new WaitForSeconds(coolDown);
            secondAttackAvailable = true;
        }
    }
}
