using System.Collections;
using TMPro;
using UnityEngine;
using UnityEngine.UI;

public class DeathAnimation : MonoBehaviour
{
    private Players player;

    // Ecran de mort
    private Transform screen;

    // Texte de mort
    private Transform text;

    // Bouton pour quitter
    private Transform button;

    // Taille minimum du texte de mort
    private float minFontSize = 60f;

    // Taille maximum du texte de mort
    private float maxFontSize = 70f;

    // vitesse à laquelle évolue la taille du texte
    // 0 < iteratorFontSize < 1
    private float iteratorFontSize = .1f;

    public void Init()
    {
        // On récupère le player
        player = GameObject.Find("player" + Lobby.login).GetComponent<PlayerController>().player;


        // L'idée est d'utiliser des events, afin d'afficher l'écran de mort uniquement à la mort du personnage
        // Ici, cherche à récupérer les events de la classe Players
        Players UIEvent = player;

        // On selectionne les différents events qui nous seront utiles,
        // et on leurs attribue les fonctions associées
        UIEvent.onDeath += DeathUI;


        // this.gameObject permet de récupérer l'objet auquel le script est associé (dans notre cas, le Canvas de l'IHM)
        screen = this.gameObject.transform.Find("DeathScreen");
        text = screen.Find("DeathText");
        button = screen.Find("SpectateButton");


        // Définit la fonction à appeler lorsqu'on clique sur le bouton
        button.GetComponent<Button>().onClick.AddListener(ButtonQuitOnClick);


        // Désactive l'écran de mort du personnage
        screen.gameObject.SetActive(false);
        // Désactive le texte de mort du personnage
        text.gameObject.SetActive(false);
        // Rend le texte de mort transparent
        text.GetComponent<TMP_Text>().color = new Color(0, 0, 0, 0);
        // Rend le bouton transparent
        button.gameObject.SetActive(false);
    }

    // Affichage de l'écran de mort
    public void DeathUI()
    {
        // On désactive tous les enfants du Canvas
        for (int i = 0; i < this.gameObject.transform.childCount; i++)
        {
            this.gameObject.transform.GetChild(i).gameObject.SetActive(false);
        }

        StartCoroutine(FadeScreen());
    }

    // Permet de faire fondre l'écran de mort du personnage
    public IEnumerator FadeScreen()
    {
        yield return new WaitForSeconds(1);

        screen.gameObject.SetActive(true);

        // A chaque frame, on augmente petit à petit l'alpha de la couleur de l'écran
        for (float i = 0; i <= 1; i += Time.deltaTime)
        {
            screen.GetComponent<Image>().color = new Color(0, 0, 0, i);
            yield return null;
        }

        // On fixe la valeur de l'alpha à 1 (= 255)
        // A la sortie de boucle, on est pas sûr que l'alpha soit bien à 1, donc on fixe la valeur manuellement
        screen.GetComponent<Image>().color = new Color(0, 0, 0, 1);
        
        StopCoroutine(FadeScreen());
        StartCoroutine(DeathText());
    }

    // Anime le texte :
    //      - 1ère animation : on augmente progessivement la taille de la police et l'alpha du texte (fait simultanément)
    //      - 2ème animation : on monte le texte et on affiche le bouton
    public IEnumerator DeathText()
    {
        text.gameObject.SetActive(true);

        // Correspond au pas pour augmenter l'alpha aussi vite que la taille du texte
        // Permet d'avoir une augmentation simultanée de l'alpha du texte et de la taille du texte
        float iteratorColor = 1f / ((maxFontSize - minFontSize) / iteratorFontSize);

        float currentFontSize = minFontSize;
        float currentColor = 0;

        // A chaque frame :
        //      - On augmente la taille de la police
        //      - On augmente l'alpha du texte
        while (currentFontSize < maxFontSize)
        {
            currentFontSize += iteratorFontSize;
            currentColor += iteratorColor;

            text.GetComponent<TMP_Text>().fontSize = currentFontSize;
            text.GetComponent<TMP_Text>().color = new Color(1, 0, 0, currentColor);

            yield return null;
        }

        // On fixe la taille de la police à "maxFontSize" et la valeur de l'alpha à 1 (= 255)
        // A la sortie de boucle, on est pas sûr qu'ils ont ces valeurs, donc on fixe la valeur manuellement
        text.GetComponent<TMP_Text>().fontSize = maxFontSize;
        text.GetComponent<TMP_Text>().color = new Color(1, 0, 0, 1);

        // Attend 1 seconde entre les 2 animations
        yield return new WaitForSeconds(1);

        for (float i = 0; i < 50f; i++)
        {
            text.position = (this.gameObject.transform.position + new Vector3(0, i, 0));
            yield return null;
        }

        button.gameObject.SetActive(true);

        StopCoroutine(DeathText());
    }

    // Fonction qu'on appelle lorsqu'on clique sur le bouton
    public void ButtonQuitOnClick()
    {
        button.gameObject.SetActive(false);
        QuitGame();
    }

    // quitte la partie en changeant de scène
    public void QuitGame()
    {
        Game.LeaveGame();

        StopAllCoroutines();
        
        // suppression de tous les gameObject inutile du DontDestroyOnLoad
        Destroy(GameObject.Find("Canvas"));
        Destroy(GameObject.Find("CameraManager"));
        Destroy(GameObject.Find("MinimapCamera"));
        Destroy(GameObject.Find("player" + Lobby.login));
        Destroy(GameObject.Find("Main_Camera"));

        GameObject[] enemies = GameObject.FindGameObjectsWithTag("enemy");
        GameObject[] players = GameObject.FindGameObjectsWithTag("player");
        for (int i = 0; i < enemies.Length; i++) // supprimer tous les ennemies créé
        {
            Destroy(enemies[i]);
        }
        for (int i = 0; i < players.Length; i++) // supprimer tous les ennemies créé
        {
            Destroy(players[i]);
        }
    }
}