using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Audio;

public class InGameSettings : MonoBehaviour
{
    [Header("Les différentes scènes")]
    [SerializeField] GameObject settingsMenu;


    [Header("Les boutons de la scène")]
    [SerializeField] Button settingsButton;
    [SerializeField] Button closeButton;


    [Header("Les sliders")]
    // Slider des musiques de combat
    public Slider musicSoundBar;
    // Slider des sons des personnages (épée, flèche...)
    public Slider characterSoundBar;
    // Slider des sons d'environnement (coffre, pièces...)
    public Slider environmentSoundBar;


    [Header("Paramètres de son")]
    // Audio mixer des musiques de combat
    public AudioMixer musicMixer;
    // Audio mixer des sons des personnages (épée, flèche...)
    public AudioMixer characterMixer;
    // Audio mixer des sons d'environnement (coffre, pièces...)
    public AudioMixer environmentMixer;


    void Start()
    {
        settingsButton.onClick.AddListener(EnableSettingsMenu);
        closeButton.onClick.AddListener(DisableSettingsMenu);

        // Variable temporaire pour récupérer le volume des différents Audio mixer
        float volumeValue = 0;

        // On définit la valeur du Slider musicSoundBar en fonction du volume
        musicMixer.GetFloat("MusicVolume", out volumeValue);
        musicSoundBar.value = volumeValue;

        // On définit la valeur du Slider characterSoundBar en fonction du volume
        characterMixer.GetFloat("CharacterVolume", out volumeValue);
        characterSoundBar.value = volumeValue;

        // On définit la valeur du Slider environmentSoundBar en fonction du volume
        environmentMixer.GetFloat("EnvironmentVolume", out volumeValue);
        environmentSoundBar.value = volumeValue;
    }

    void Enable()
    {
        settingsMenu.SetActive(false);
    }

    void Update()
    {
        if (Input.GetKeyDown(KeyCode.Escape))
        {
            // Si le menu des paramètres est activé, le désactive 
            if (settingsMenu.activeSelf)
                closeButton.onClick?.Invoke();

            // Si le menu des paramètres est désactivé, l'active 
            else
                settingsButton.onClick?.Invoke();
        }
    }


    // Active le menu des paramètres
    private void EnableSettingsMenu()
    {
        // Si le menu des paramètres est activé, le désactive
        // Utile car même lorsque le menu des paramètres est activé, le bouton est visible
        if (settingsMenu.activeSelf)
            closeButton.onClick?.Invoke();

        else
            settingsMenu.SetActive(true);
    }


    // Désactive le menu des paramètres
    private void DisableSettingsMenu()
    {
        settingsMenu.SetActive(false);
    }


    // Modifie le volume des musiques de combat
    // https://www.youtube.com/watch?v=YOaYQrN1oYQ
    public void SetVolumeMusic(float volumeM)
    {
        if (volumeM == musicSoundBar.minValue)
            volumeM = -80;

        musicMixer.SetFloat("MusicVolume", volumeM);
    }

    // Modifie le volume des sons des personnages (épée, flèche...)
    public void SetVolumeCharacter(float volumeC)
    {
        if (volumeC == characterSoundBar.minValue)
            volumeC = -80;

        characterMixer.SetFloat("CharacterVolume", volumeC);
    }

    // Modifie le volume des sons d'environnement (coffre, pièces...)
    public void SetVolumeEnvironment(float volumeE)
    {
        if (volumeE == environmentSoundBar.minValue)
            volumeE = -80;

        environmentMixer.SetFloat("EnvironmentVolume", volumeE);
    }
}
