using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Minimap : MonoBehaviour
{
    private Transform player;


    // Listes des différents éléments qu'on désire afficher sur la minimap
    public GameObject[] playersGameObject;
    public GameObject[] enemiesGameObject;
    public GameObject[] chestsGameObject;

    public void Init()
    {
        DontDestroyOnLoad(gameObject);
        player = GameObject.Find("player" + Lobby.login).transform;

        // on récupère tous les éléments en fonction des tags
        playersGameObject = GameObject.FindGameObjectsWithTag("player");

        SetIconMiniMap(playersGameObject, Color.green, 1f);
    }


    public void refreshMinimap()
    {
        // on récupère tous les éléments en fonction des tags
        enemiesGameObject = GameObject.FindGameObjectsWithTag("enemy");
        chestsGameObject = GameObject.FindGameObjectsWithTag("chest");

        SetIconMiniMap(enemiesGameObject, "WhiteSkull", 1f);
        SetIconMiniMap(chestsGameObject, "Chest1", 30f);
    }

    void LateUpdate()
    {
        if (player != null)
        {
            // Met à jour la position de l'image, afin qu'elle suive l'objet
            Vector3 newPosition = player.position;
            newPosition.y = transform.position.y;
            transform.position = newPosition;
        }
    }

    // Surchage de fonction
    // Sprite par défaut : un cercle
    public static void SetIconMiniMap(GameObject[] list, Color color, float scale)
    { SetIconMiniMap(list, color, "Circle", scale); }


    // Surcharge de fonction
    // Couleur par défaut : blanche, aucune transparence
    public static void SetIconMiniMap(GameObject[] list, string src, float scale)
    { SetIconMiniMap(list, new Color(255, 255, 255, 255), src, scale); }


    // Créé une image qui sera affichée sur la minimap pour représenter les éléments de 'list'
    // Ce programme ajoute une image à la prefab des éléments de 'list'. Si vous lancez le jeu et que vous visionnez la Scène vous la verrez
    public static void SetIconMiniMap(GameObject[] list, Color color, string src, float scale)
    {
        foreach (var item in list)
        {
            // Créé l'élément qui sera affiché sur la minimap
            GameObject minimapIcon = new GameObject("MinimapIcon");

            // Ajoute un élément permettant de faire référencer un sprite, et sa couleur
            minimapIcon.AddComponent<RectTransform>();
            minimapIcon.AddComponent<SpriteRenderer>().sprite = Resources.Load<Sprite>("UI/Sprites/" + src);
            minimapIcon.GetComponent<SpriteRenderer>().color = color;

            // Place l'élément comme désiré, donc au dessus du joueur (un peu comme un parapluie)
            minimapIcon.transform.localScale = new Vector2(scale, scale);
            minimapIcon.transform.rotation = Quaternion.AngleAxis(90, Vector3.right);
            minimapIcon.transform.position = item.transform.position + new Vector3(0, 1, 0);

            // Défini le layer de l'élément
            // /!\ Très important pour le rendre visible sur la minimap /!\
            minimapIcon.layer = LayerMask.NameToLayer("minimap");

            // Défini le parent de l'élément
            minimapIcon.transform.SetParent(item.transform);
        }
    }

}