using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Networking;
using Newtonsoft.Json;
using UnityEngine.UI;
using TMPro;
using UnityEngine.EventSystems;

public class SendRequestLogin : MonoBehaviour
{
    [Header("Les différentes scènes")]
    [SerializeField] GameObject loginMenu;
    [SerializeField] GameObject mainMenu;
    [SerializeField] GameObject createAccountMenu;
    [SerializeField] GameObject settingsMenu;
    [SerializeField] GameObject lobbyMenu;


    [Header("Les boutons de la scène")]
    [SerializeField] Button buttonConnect;
    [SerializeField] Button buttonReturn;
    [SerializeField] Button buttonCreateAccount;
    [SerializeField] Button buttonSettings;


    [Header("Message d'erreur")]
    public GameObject errorMessage;


    [Header("Champs de saisi des données")]
    public GameObject userNameField;
    public GameObject passwordField;


    private string userName = "";
    private string password = "";


    // Liste des champs de saisi de la scène
    private List<GameObject> fields;

    public void Start()
    {
        // Défini les actions des boutons
        buttonConnect.onClick.AddListener(Click);
        buttonReturn.onClick.AddListener(delegate { GoFromMenuToMenu(loginMenu, mainMenu); });
        buttonCreateAccount.onClick.AddListener(delegate { GoFromMenuToMenu(loginMenu, createAccountMenu); });
        buttonSettings.onClick.AddListener(GoToSettingsMenu);

        // rempli la liste de champs de saisi
        fields = new List<GameObject> { userNameField, passwordField };
    }

    public void Update()
    {
        // Lorsqu'on presse la touche tab, change le champ de saisi sélectionné
        if (Input.GetKeyDown(KeyCode.Tab))
        {
            EventSystem.current.SetSelectedGameObject(fields[(fields.IndexOf(EventSystem.current.currentSelectedGameObject) + 1) % fields.Count]);
        }

        // Lorsqu'on presse la touche entrée, simule le bouton
        if (Input.GetKeyDown(KeyCode.Return))
        {
            buttonConnect.onClick?.Invoke();
        }

        // Lorsqu'on presse la touche echap, retourne à l'accueil
        if (Input.GetKeyDown(KeyCode.Escape))
        {
            buttonReturn.onClick?.Invoke();
        }
    }

    // Déclanché dès que l'objet est activé (= SetActive(true))
    void OnEnable()
    {
        errorMessage.SetActive(false);
        userNameField.GetComponent<TMP_InputField>().text = "";
        passwordField.GetComponent<TMP_InputField>().text = "";
    }


    // Récupère les informations de connexion saisi par le joueur
    public void Click()
    {
        errorMessage.SetActive(false);

        // Permet d'obtenir les informations contenues dans les champs de saisi
        userName = userNameField.GetComponent<TMP_InputField>().text;
        password = passwordField.GetComponent<TMP_InputField>().text;

        if (userName != "" && password != "")
            StartCoroutine(GetText());
        else
            errorMessage.SetActive(true);
    }

    // Communique les identifiants de connexion du joueur à la BDD
    IEnumerator GetText()
    {
        // Créé un formulaire d'envoie spécifique pour être compris par la BDD
        WWWForm form = new WWWForm();
        form.AddField("userName", userName);
        form.AddField("password", password);

        // On communique les informations
        using (UnityWebRequest webRequest = UnityWebRequest.Post("https://mai-projet-integrateur.u-strasbg.fr/vmProjetIntegrateurgrp6-0/login.php", form))
        {
            webRequest.certificateHandler = new CertificateOverride();
            yield return webRequest.SendWebRequest();

            // La connexion au serveur a echoue
            if (webRequest.result == UnityWebRequest.Result.ConnectionError || webRequest.result == UnityWebRequest.Result.ProtocolError)
            {
                Debug.Log(webRequest.error);
            }
            // La connexion au serveur a reussi
            else
            {
                if (webRequest.isDone)
                {
                    // Converti la réponse en JSON
                    var requestRes = JsonConvert.DeserializeObject<Dictionary<string, string>>(webRequest.downloadHandler.text);
                    Debug.Log(webRequest.downloadHandler.text);


                    // Les identifiants renseignes par le joueur sont corrects
                    if (requestRes["isSuccessful"] == "1")
                    {
                        PlayerInfo.SetUserName(userName);
                        PlayerInfo.SetConnection(true);
                        GoFromMenuToMenu(loginMenu, lobbyMenu);
                    }


                    // Les identifiants renseignes par le joueur sont erronnes
                    else if (requestRes["isSuccessful"] == "0")
                    {
                        errorMessage.SetActive(true);
                    }
                }

            }
        }
    }


    private void GoFromMenuToMenu(GameObject menuA, GameObject menuB)
    {
        menuA.SetActive(false);
        menuB.SetActive(true);
    }

    private void GoToSettingsMenu()
    {
        settingsMenu.SetActive(true);
    }
}