using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Audio;
using UnityEngine.SceneManagement;
using TMPro;
using UnityEngine.Networking;
using Newtonsoft.Json;

public class SettingMenu : MonoBehaviour
{
    [Header("Le canvas")]
    [SerializeField] GameObject canvas;


    [Header("Les différentes scènes")]
    [SerializeField] GameObject mainMenu;
    [SerializeField] GameObject settingsMenu;


    [Header("Les boutons de la scène")]
    [SerializeField] Button buttonReturn;
    [SerializeField] Button buttonDisconnect;


    [Header("Les sliders de son")]
    // Slider des musiques de combat
    public Slider musicSoundBar;
    // Slider des sons des personnages (épée, flèche...)
    public Slider characterSoundBar;
    // Slider des sons d'environnement (coffre, pièces...)
    public Slider environmentSoundBar;


    [Header("Paramètres de son")]
    // Audio mixer des musiques de combat
    public AudioMixer musicMixer;
    // Audio mixer des sons des personnages (épée, flèche...)
    public AudioMixer characterMixer;
    // Audio mixer des sons d'environnement (coffre, pièces...)
    public AudioMixer environmentMixer;


    [Header("Le dropdown")]
    // Le Dropdown pour séléctionner le serveur de jeu
    public TMP_Dropdown serverSelection;


    [Header("Description serveur")]
    // Description du serveur Muon
    public TMP_Text muonDescription;
    // Description du serveur Unistra
    public TMP_Text unistraDescription;

    [Header("Statistiques joueur")]
    // Wrapper de l'affichage des stats
    public GameObject playerStats;
    // Statistiques joueur
    public TMP_Text playerStatsTxt;


    void Start()
    {
        // Défini les actions des boutons
        buttonReturn.onClick.AddListener(CloseMenu);
        buttonDisconnect.onClick.AddListener(Disconnection);

        // Variable temporaire pour récupérer le volume des différents Audio mixer
        float volumeValue = 0;

        // On définit la valeur du Slider musicSoundBar en fonction du volume
        musicMixer.GetFloat("MusicVolume", out volumeValue);
        musicSoundBar.value = volumeValue;

        // On définit la valeur du Slider characterSoundBar en fonction du volume
        characterMixer.GetFloat("CharacterVolume", out volumeValue);
        characterSoundBar.value = volumeValue;

        // On définit la valeur du Slider environmentSoundBar en fonction du volume
        environmentMixer.GetFloat("EnvironmentVolume", out volumeValue);
        environmentSoundBar.value = volumeValue;
    }

    void OnEnable()
    {
        if (PlayerInfo.GetConnection())
        {
            buttonDisconnect.gameObject.SetActive(true);
            playerStats.gameObject.SetActive(true);
            StartCoroutine(GetStats());
        }
        else
        {
            buttonDisconnect.gameObject.SetActive(false);
            playerStats.gameObject.SetActive(false);
        }

        serverSelection.value = (ConnectionGame.currentServer == ConnectionGame.serverMuonAddress ? 0 : 1);
    }

    // Modifie le volume des musiques de combat
    // https://www.youtube.com/watch?v=YOaYQrN1oYQ
    public void SetVolumeMusic(float volumeM)
    {
        if (volumeM == musicSoundBar.minValue)
            volumeM = -80;

        musicMixer.SetFloat("MusicVolume", volumeM);
    }

    // Modifie le volume des sons des personnages (épée, flèche...)
    public void SetVolumeCharacter(float volumeC)
    {
        if (volumeC == characterSoundBar.minValue)
            volumeC = -80;

        characterMixer.SetFloat("CharacterVolume", volumeC);
    }

    // Modifie le volume des sons d'environnement (coffre, pièces...)
    public void SetVolumeEnvironment(float volumeE)
    {
        if (volumeE == environmentSoundBar.minValue)
            volumeE = -80;

        environmentMixer.SetFloat("EnvironmentVolume", volumeE);
    }

    private void CloseMenu()
    {
        settingsMenu.SetActive(false);
    }

    private void Disconnection()
    {
        PlayerInfo.SetConnection(false);
        for (int i = 0; i < canvas.transform.childCount; i++)
        {
            canvas.transform.GetChild(i).gameObject.SetActive(false);
        }
        mainMenu.SetActive(true);
    }

    public void OnSelectionServer()
    {
        switch (serverSelection.value)
        {
            // Serveur Muon
            case 0:
                muonDescription.gameObject.SetActive(true);
                unistraDescription.gameObject.SetActive(false);

                FindObjectOfType<ConnectionGame>().DeconnectionServer();
                ConnectionGame.currentServer = ConnectionGame.serverMuonAddress;
                FindObjectOfType<ConnectionGame>().ConnectionServer();
                break;

            // Serveur Unistra
            case 1:
                unistraDescription.gameObject.SetActive(true);
                muonDescription.gameObject.SetActive(false);

                FindObjectOfType<ConnectionGame>().DeconnectionServer();
                ConnectionGame.currentServer = ConnectionGame.serverUnistraAddress;
                FindObjectOfType<ConnectionGame>().ConnectionServer();
                break;

            default:
                break;
        }
    }

    IEnumerator GetStats()
    {
        // Créé un formulaire d'envoie spécifique pour être compris par la BDD
        WWWForm form = new WWWForm();
        form.AddField("userName", PlayerInfo.GetUserName());

        // On communique les informations
        using (UnityWebRequest webRequest = UnityWebRequest.Post("https://mai-projet-integrateur.u-strasbg.fr/vmProjetIntegrateurgrp6-0/get_player_stats.php", form))
        {
            webRequest.certificateHandler = new CertificateOverride();
            yield return webRequest.SendWebRequest();

            // La connexion au serveur a echoue
            if (webRequest.result == UnityWebRequest.Result.ConnectionError || webRequest.result == UnityWebRequest.Result.ProtocolError)
            {
                Debug.Log(webRequest.error);
            }
            // La connexion au serveur a reussi
            else
            {
                if (webRequest.isDone)
                {
                    // Converti la réponse en JSON
                    var requestRes = JsonConvert.DeserializeObject<Dictionary<string, string>>(webRequest.downloadHandler.text);

                    // Les identifiants renseignes par l'utilisateur sont corrects
                    if (requestRes["isSuccessful"] == "1")
                    {
                        string statStr;
                        if (PlayerInfo.GetUserName().Length < 10)
                        {
                            statStr = PlayerInfo.GetUserName() + ", vous accumulez " + requestRes["nbWins"] + " victoires et " + requestRes["nbCoins"] + " pièces d'or";
                        }
                        else
                        {
                            statStr = "Vous accumulez " + requestRes["nbWins"] + " victoires et " + requestRes["nbCoins"] + " pièces d'or";
                        }
                        
                        playerStatsTxt.SetText(statStr);
                    }
                    else
                    {
                        playerStats.gameObject.SetActive(false);
                    }
                }
            }
        }
    }

}
