<?php
// Code source de création d'utilisateur côté serveur
// Réceptionne la requête du client et lis le nom d'utilisateur et le mot de passe
// Déchiffre les champs et hash le mot de passe
// Stocke les informations dans la base de données dungeon_escape dans la table Users
include 'mysql_connect.php';
include 'check_data_format.php';

// Les champs sont chiffrés et doivent être déchiffrés avant
// de s'assurer de leur bon format, c'est à dire : 
// - la longueur doit se situer entre 3 et 50 caractères
// - les champs ne doivent pas contenir de 'whitespace' (e.g. un espace ou '\n')
$client_login = htmlspecialchars($_POST['userName']);
$client_password = htmlspecialchars($_POST['password']);
$client_pass_hash = password_hash($client_password, PASSWORD_BCRYPT);

// TODO RSA déchiffrer les champs

check_format($client_login, $client_password);

// Préparation des commandes SQL, l'affectation par des paramètres empèche les injections SQL
$verify_login = $db_connection->prepare('select * from users where login=?');
$verify_login->bind_param('s', $client_login);
$verify_login->execute();

$verify_res = $verify_login->get_result();
if ($verify_res->num_rows > 0) {
	$errorMessage = 'This login is already in use.';
	echo json_encode(array('isSuccessful' => 0, 'errorCode' => 500, 'errorMessage' => $errorMessage));
	exit(500);
} else {
	if ($create_sql = $db_connection->prepare('insert into users (login, password) values (?, ?)')) {
		$create_sql->bind_param('ss', $client_login, $client_pass_hash);
		$create_res = $create_sql->execute();

		if ($create_res) {
			$returnMessage = 'Account created successfully';
			echo json_encode(array('isSuccessful' => 1, 'returnMessage' => $returnMessage, 'errorCode' => 0, 'errorMessage' => 'No errors, task succeeded.'));
			exit(0);
		} else {
			$errorMessage = $db_connection->error;
			echo json_encode(array('isSuccessful' => 0, 'errorCode' => 600, 'errorMessage' => $errorMessage));
			exit(600);
		}
	}
}

$db_connection->close();
?>
