<?php
// Code source pour l'authentification côté serveur
// Réceptionne la requête du client et lis le nom d'utilisateur et le mot de passe
// Déchiffre les champs et hash le mot de passe
// Cherche le mot de passe associé au nom d'utilisateur dans la base de données
// DungeonEscape, dans la table Users
// Vérifie que le hash du mot de passe de la requête et celui de la
// base de données correspondent et renvoie une requête au client
include 'mysql_connect.php';
include 'check_data_format.php';

// Les champs sont chiffrés et doivent être déchiffrés avant
// de s'assurer de leur bon format, c'est à dire : 
// - la longueur doit se situer entre 3 et 50 caractères
// - les champs ne doivent pas contenir de 'whitespace' (e.g. un espace ou '\n')
$client_login = htmlspecialchars($_POST['userName']);
$client_password = htmlspecialchars($_POST['password']);

// TODO RSA déchiffrer les champs

check_format($client_login, $client_password);

// Préparation des commandes SQL, l'affectation par des paramètres empèche les injections SQL
$login_sql = $db_connection->prepare('select password from users where login = ?');
$login_sql->bind_param('s', $client_login);
$login_sql->execute();

$login_res = $login_sql->get_result();
if ($login_res->num_rows > 0) {
	//Le nom d'utilisateur existe dans la BDD
	$login_row = $login_res->fetch_assoc();
	$db_password = $login_row['password'];

	if (password_verify($client_password, $db_password)) {
		$returnMessage = 'Authentification was successful.';
		echo json_encode(array('isSuccessful' => 1, 'errorCode' => 0, 'returnMessage' => $returnMessage, 'errorMessage' => 'No errors, task succeeded.'));
	} else {
		$errorMessage = 'Authentication failed, password did not match.';
		echo json_encode(array('isSuccessful' => 0, 'errorCode' => 300, 'errorMessage' => $errorMessage));
	}
} else {
	$errorMessage = 'Login does not exist in database.';
	echo json_encode(array('isSuccessful' => 0, 'errorCode' => 400, 'errorMessage' => $errorMessage));
}

$db_connection->close();
?>
