var container = document.querySelector("#unity-container");
var canvas = document.querySelector("#unity-canvas");
var loadingBar = document.querySelector("#unity-loading-bar");
var progressBarFull = document.querySelector("#unity-progress-bar-full");
var fullscreenButton = document.querySelector("#unity-fullscreen-button");
var warningBanner = document.querySelector("#unity-warning");

// Shows a temporary message banner/ribbon for a few seconds, or
// a permanent error message on top of the canvas if type=='error'.
// If type=='warning', a yellow highlight color is used.
// Modify or remove this function to customize the visually presented
// way that non-critical warnings and error messages are presented to the
// user.
function unityShowBanner(msg, type) {
  function updateBannerVisibility() {
    warningBanner.style.display = warningBanner.children.length ? 'block' : 'none';
  }
  var div = document.createElement('div');
  div.innerHTML = msg;
  warningBanner.appendChild(div);
  if (type == 'error') div.style = 'background: red; padding: 10px;';
  else {
    if (type == 'warning') div.style = 'background: yellow; padding: 10px;';
    setTimeout(function() {
      warningBanner.removeChild(div);
      updateBannerVisibility();
    }, 5000);
  }
  updateBannerVisibility();
}

var buildUrl = "Build";
var loaderUrl = buildUrl + "/web_client.loader.js";
var config = {
  dataUrl: buildUrl + "/fd65988eae5791106de5bc52efc20cb4.data",
  frameworkUrl: buildUrl + "/f07279a7e6e5adb860ced54fc4f0a9fe.js",
  codeUrl: buildUrl + "/49a6a1110832e4e14e82b37f05f0daa9.wasm",
  streamingAssetsUrl: "StreamingAssets",
  companyName: "DefaultCompany",
  productName: "Dungeon Escape",
  productVersion: "1.0",
  showBanner: unityShowBanner,
};

// By default Unity keeps WebGL canvas render target size matched with
// the DOM size of the canvas element (scaled by window.devicePixelRatio)
// Set this to false if you want to decouple this synchronization from
// happening inside the engine, and you would instead like to size up
// the canvas DOM size and WebGL render target sizes yourself.
// config.matchWebGLToCanvasSize = false;

if (/iPhone|iPad|iPod|Android/i.test(navigator.userAgent)) {
  // Mobile device style: fill the whole browser client area with the game canvas:

  var meta = document.createElement('meta');
  meta.name = 'viewport';
  meta.content = 'width=device-width, height=device-height, initial-scale=1.0, user-scalable=no, shrink-to-fit=yes';
  document.getElementsByTagName('head')[0].appendChild(meta);
  container.className = "unity-mobile";
  canvas.className = "unity-mobile";

  // To lower canvas resolution on mobile devices to gain some
  // performance, uncomment the following line:
  // config.devicePixelRatio = 1;

  unityShowBanner('WebGL builds are not supported on mobile devices.');
} else {
  // Desktop style: Render the game canvas in a window that can be maximized to fullscreen:

  canvas.style.width = "960px";
  canvas.style.height = "600px";
}

loadingBar.style.display = "block";

var script = document.createElement("script");
script.src = loaderUrl;
script.onload = () => {
  createUnityInstance(canvas, config, (progress) => {
    progressBarFull.style.width = 100 * progress + "%";
  }).then((unityInstance) => {
    loadingBar.style.display = "none";
    fullscreenButton.onclick = () => {
      unityInstance.SetFullscreen(1);
    };
  }).catch((message) => {
    alert(message);
  });
};
document.body.appendChild(script);

// script for the rest of site

//get all the li elements of the menu
const gameLi = document.getElementById("gameLi");
const descriptionLi = document.getElementById("descriptionLi");
const donwloadLi = document.getElementById("downloadLi");

//get the div that contains the elements of the menu
const gameDiv = document.getElementById("gameDiv");
const descriptionDiv = document.getElementById("descriptionDiv");
const donwloadDiv = document.getElementById("downloadDiv");

gameLi.addEventListener("click", () => { 
  if(gameDiv.classList.contains("hidden")){
    gameDiv.classList.remove("hidden");
    gameLi.classList.add("selected");
    descriptionDiv.classList.add("hidden");
    descriptionLi.classList.remove("selected");
    donwloadDiv.classList.add("hidden");
    donwloadLi.classList.remove("selected");
  }
});
descriptionLi.addEventListener("click", () => {
  if(descriptionDiv.classList.contains("hidden")){
    descriptionDiv.classList.remove("hidden");
    descriptionLi.classList.add("selected");
    gameDiv.classList.add("hidden");
    gameLi.classList.remove("selected");
    donwloadDiv.classList.add("hidden");
    donwloadLi.classList.remove("selected");
  }
 });
donwloadLi.addEventListener("click", () => {
  if(donwloadDiv.classList.contains("hidden")){
    donwloadDiv.classList.remove("hidden");
    donwloadLi.classList.add("selected");
    gameDiv.classList.add("hidden");
    gameLi.classList.remove("selected");
    descriptionDiv.classList.add("hidden");
    descriptionLi.classList.remove("selected");
 }});
