

# Table of contents

1. [Dependency's](#Dependency's)
2. [Dedicated server information](#Dedicated\server\information)
3. [Server deployment](##Server\deployment)
4. [Data structures](#Data\structures)
	1. [Paquet](#Paquet)
	2. [lobbyData](#lobbyData)
	3. [players](#players)
5. [Server behavior and error codes](#Server\behavior\and\error\codes)
6. [Functions](#Functions)
	1. [findIndex(table, key, element)](#findIndex(table,key,element))
	2. [removePlayer(socket)](#removePlayer(socket))
	3. [extractLogins(players)](#extractLogins(players))
	4. [printServerData()](#printServerData())
7. [Websocket flow](#Websocket\flow)
	1. [Communication](#Communication)
	2. [Data checking](#Data\checking)
	3. [Lobby Creation](#Lobby\Creation)
	4. [Lobby Joining](#Lobby\Joining)
	5. [Data Relay](#Data\Relay)
	6. [Disconnection](#Disconnection)
	7. [Nothing to see](#Nothing\to\see)

## Dependency's

- node (``` sudo apt install nodejs ```)
- npm (``` sudo apt install npm ```): 
  - ws (``` npm install ws ```)

## Dedicated server information

	URL: 185.155.93.105:16000 (not working for now)
    URL: ailugensvps.ovh:10000 (temporary)

## Server deployment

1. Copy server.js and the unity binaries in the folder you want
2. initialize the node module :
   1. in a terminal open the folder
   2. initialize the node module : ```npm init```
   3. you don't have to fill any data except for the entry point : ```server.js```
   4. install *ws*: ```npm install ws``` 
3. to start the server execute the command: ```node ./server.js```
   1. if you want to start it in debug mode: ```node ./server.js debug```

## Data structures

### Paquet
| login : String | roomCode : String | createLobby : Bool | inLobby : Bool | isReady : Bool | playersInLobby : String array | errorCode : Int | errorString : String | data : *1 |
| -------------- | ----------------- | ------------------ | -------------- | -------------- | ----------------------------- | --------------- | -------------------- | --------- |

*1* : data contain any data the client needs ( it isn't read by the server)

### lobbyData

| roomCode : String | nbPlayers :  int | players : players |
| ----------------- | ---------------- | ----------------- |

### players

| login : String | socket : websocket |
| ------------- | --------------------|

## Server behavior and error codes

#### General error code : 1

\* : data coming from  client
&rarr; : data return from server
__ : value not modified 

data expected for cases :

| case                                   | login | roomCode | creatLobby | inLobby | isReady    | errorCode  | data |
| -------------------------------------- | ----- | -------- | ---------- | ------- | ---------- | ---------- | ---- |
| \* packet corrupt | __ | __ | __ | __ | __ | __ | __ |
| &rarr; error | __ | __ | __ | __ | __ | 1 | __ |      
| \* lobby creation                      | __    | expected | true       | false   | __         | none       | __   |
| &rarr; succes                          | __    | __       | false      | __      | __         | none       | __   |
| &rarr; failure: is in lobby            | __    | none     | true       | __      | __         | error : 11 | __   |
| \* join                                | __    | expected | false      | false   | __         | none       | __   |
| &rarr; succes                          | __    | __       | __         | true    | __         | none       | __   |
| &rarr; failure: lobby not found        | __    | none     | __         | __      | __         | error : 22 | __   |
| &rarr; failure: lobby full             | __    | none     | __         | __      | __         | error : 23 | __   |
| &rarr; success: lobby quitted             | __    | none     | __         | __      | __         | error : 24 (local), error : 40 (other players)| __   |
| \* deconection                         | __    | __       | __         | __      | __         | __         | __   |
| &rarr; clean or dirty (or lobby quitting)                         | __    | __       | __         | __      | __         | error: 40  | __   |
| \* isReady                             | __    | __       | __         | __      | true/false | __         | __   |
| &rarr; warning : one player's status changed                         | __    | __       | __         | __      | __         | error: 30  | __   |

<!--     | | | | | | | | |           

-->

## Functions

### findIndex( table, key, element)

Function returning the index of an element in a table.

	in table   : table of any type
	in key     : searched type
	in element : the element to be searched
	out int    : the index of the element or -1

### removePlayer(socket)

Function removing a player from a lobby and communication it to the rest of the lobby

	in socket              : socket of the player
	   indexLobby          : index of the lobby in wich the player is
	   indexPlayerToRemove : index of the player to remove

### extractLogins(players)

Function returning the login of a player from the couple [login, socket] stored in lobbyData[players].

	int player : socket of the targeted player
	out logins : login of the targeted player

### printServerData()

Function printing lobbyData in the console of the server.

Debug function printing all the content of lobbyData, return nothing.

## Websocket flow

Explanation of the code in each marker.

### Communication

The server parse the data received into `dataJson` and search the index of the lobby where the player is and store it in `indexLobby`.

### Data checking

Here the server check the integrity of the data the server receive.

<!--
Line 174. if statement meant to check if the data structure read is correct. If not correct send an error ```1``` to the server.
-->
### Lobby Creation

The server read createLobby. 
If `true` create the lobby if it doesn't already exist, if it already exist send an error `11` to the client. 
If `false` does nothing.

<!--
Line 209. Do a sanity check for the case someone who is in a lobby and isn't trying to create a lobby. It shouldn't be happening.

Line 220. Handler for the creation of the lobby (```"createLobby" === "true"```)
First check if the lobby already exist. If it exist send back an error code else create the lobby and put the player in it. -->

### Lobby Joining

Handling for the joining process of the player.
If `lobbyIndex =! -1`, `inLobby == false` and `nbPlayer < 4` put the player in the lobby and send the appropriate data's for confirmation.
If the lobby ins't found, send an error `22` to the client.
If `nbPlayer >= 4` send an error `23` to the client.
<!--
Line 261. Verification that the room exist, `lobbyIndex !=-1`.

Line 272. -->

### Data Relay

The server retransmit `dataJson` to all player in the lobby except the player who is communicating with the server.

### Disconnection

The server identifie a deconnection and remove the player from the lobby and the data of the other player of the lobby.

# ToDo

- [x] ajouter les autres joueurs du lobby dans la data 

# Demandes du groupe client

Empty for now

# Demande pour le groupe client

Empty for now

# Première connexion (donc sans login) :

- [x] Identifer le cas (login ou pas)

- [x] Appel de la fonction pour créer un login (bdd et tout)

- [x] Renvoier au client son login

# Connexion au serveur (avec login)

## Création de lobby (why bdd ?)

- [x] Identifier le cas
  
  - [x] Envoi une erreur si au client si le lobby existe deja

- [x] Appel de la fonction pour créer un lobby
  
  - [x] insertion dans la liste de lobby locals

## Rejoindre le lobby (why bdd ?)

- [x] Identifier le cas

- [x] Récupérer le lobby et ses données (joueurs, ...)

- [x] Associer le lobby aux données du client (structure client)

## Quitter le lobby

- [x] Identifier le cas
  
  - [x] deconnexion sauvage
  
  - [x] deconnexion propre

- [x] Avertir les autres joueurs de la partie de la déconnexion

# Gestion de client maitre (IA)

- [ ] Identifier le cas

- [ ] Relayer les data IA

# Gestion déroulement de la partie

- [x] Identifier le cas

- [x] Identifier le lobby

- [x] Relayer les structures de données aux clients dans le même lobby

## Extras :

- [x] Mode debug activable ou non (anti-spam de console.log)
